/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGenr audio stuff

#include "sbagenr.h"

#define	WINERR_MAX	(256)	/* max size of win sound error msg. */

#ifdef WIN_AUDIO
static int aud_whprep_cnt;	// prepared audio header counter
#endif

/* Setup audio device
*/
void setup_device(void)
{
 out_file = NULL;
 out_fd = -1;
 
 // Handle output to files and pipes
 if(opt_O || opt_o)
 {
  if(opt_O)
  {
   out_file = stdout;
   out_fd = (int)fileno(out_file);		// stdout
  }
  else
  {
   if(!(out_file = fopen(opt_o, "wb")))
    scr_error("Can't open \"%s\", errno %d", opt_o, errno);
   out_fd = (int)fileno(out_file);
  }
  out_blen = out_rate * 2 / out_prate;	// 10 fragments a second by default

  out_blen = dpwr2(out_blen);		// Make power of two

  out_bsiz = out_blen * (out_mode? 2 : 1);
  out_bps = out_mode? 4 : 2;
  out_buf = (short *)Alloc(out_blen * sizeof(short));
  out_buf_lo = (int)(0x10000 * 1000.0 * 0.5 * out_blen / out_rate);
  out_buf_ms = out_buf_lo >> 16;
  out_buf_lo &= 0xFFFF;
  tmp_buf = (int *)Alloc(out_blen * sizeof(int));

  if(!opt_Q && !opt_W)		// Informational message for opt_W is written later
   warn("Outputting %d-bit raw audio data at %d Hz with %d-sample blocks, %d ms per block",
	out_mode? 16 : 8, out_rate, out_blen / 2, out_buf_ms);
  return;
 }

#ifdef OSS_AUDIO
 // Normal /dev/dsp output
 {
  int stereo, rate, fragsize, numfrags, enc;
  int targ_ms = 400;	// How much buffering we want, ideally
  int afmt_req, afmt;
  int test = 1;
  audio_buf_info info;
  int retry = 0;

  fragsize = 14;  // Ask for fragments of 2^14 == 16384 bytes == 4096 samples

  for(;;)
  {
   if(0 > (out_fd = open(opt_d, O_WRONLY)))
    scr_error("Can't open %s, errno %d", opt_d, errno);

   afmt = afmt_req = ((out_mode == 0)? AFMT_U8 :
	((char*)&test)[0]? AFMT_S16_LE : AFMT_S16_BE);
   stereo = 1;
   rate = out_rate;
   numfrags = (out_rate * 4 * targ_ms / 1000) >> fragsize;
   if(numfrags < 1) numfrags = 1;
   enc = (numfrags << 16) | fragsize;

   if(0 > ioctl(out_fd, SNDCTL_DSP_SETFRAGMENT, &enc) ||
	0 > ioctl(out_fd, SNDCTL_DSP_SAMPLESIZE, &afmt) ||
	0 > ioctl(out_fd, SNDCTL_DSP_STEREO, &stereo) ||
	0 > ioctl(out_fd, SNDCTL_DSP_SPEED, &rate))
    scr_error("Can't configure %s, errno %d", opt_d, errno);
       
   if(afmt != afmt_req)
    scr_error("Can't open device in %d-bit mode", out_mode? 16 : 8);
   if(!stereo)
    scr_error("Can't open device in stereo");

   out_rate = rate;

   if(-1 == ioctl(out_fd, SNDCTL_DSP_GETOSPACE, &info))
    scr_error("Can't get audio buffer info, errno %d", errno);

   if(!retry && info.fragsize != (1 << fragsize))
   {
    // We've received a different fragment size to what we asked
    // for.  This means that the numfrags calculation is wrong.
    // Re-open based on this fragsize.
    close(out_fd);
    for(fragsize = 1; (1 << fragsize) < info.fragsize; fragsize++)
     /* NOTHING */ ;
    retry = 1;
    //warn("Retrying /dev/dsp open for fragsize %d", fragsize);
    continue;
   }
   break;
  }		// for(;;)

  out_bsiz = info.fragsize;
  out_blen = out_mode? out_bsiz / 2 : out_bsiz;
  out_bps = out_mode? 4 : 2;
  out_buf = (short *)Alloc(out_blen * sizeof(short));
  out_buf_lo = (int)(0x10000 * 1000.0 * 0.5 * out_blen / out_rate);
  out_buf_ms = out_buf_lo >> 16;
  out_buf_lo &= 0xFFFF;
  tmp_buf= (int *)Alloc(out_blen * sizeof(int));

  if(!opt_Q)
   warn("Outputting %d-bit audio at %d Hz with %d %d-sample fragments, %d ms per fragment",
	out_mode? 16 : 8, out_rate, info.fragstotal, out_blen / 2, out_buf_ms);
 }	// OSS_AUDIO
#endif

#ifdef WIN_AUDIO
 // Output using Win32 calls
 {
  MMRESULT rv;
  WAVEFORMATEX fmt;
  int a;

  aud_isopen = 0;
  aud_whprep_cnt = 0;

  fmt.wFormatTag = WAVE_FORMAT_PCM;
  fmt.nChannels = 2;
  fmt.nSamplesPerSec = out_rate;
  fmt.wBitsPerSample = out_mode? 16 : 8;
  fmt.nBlockAlign = fmt.nChannels * fmt.wBitsPerSample / 8;
  fmt.nAvgBytesPerSec = out_rate * 2 * (out_mode? 2 : 1);
  fmt.cbSize = 0;
  aud_handle = NULL;

  //if (MMSYSERR_NOERROR !=
  //	waveOutOpen(&aud_handle, WAVE_MAPPER, &fmt, 0, 
  //	0L, WAVE_FORMAT_QUERY))
  // error("Windows is rejecting our audio request (%d-bit stereo, %dHz)",
  //	out_mode? 16 : 8, out_rate);

  if(MMSYSERR_NOERROR !=
	(rv = waveOutOpen(&aud_handle, WAVE_MAPPER,
	(WAVEFORMATEX *)&fmt, (DWORD)win32_audio_callback,
	(DWORD)0, CALLBACK_FUNCTION)))
  {
   char bufA[WINERR_MAX];
#ifdef UNICODE
   TCHAR bufW[WINERR_MAX];

   waveOutGetErrorText(rv, bufW, WINERR_MAX - 1);
   uni2asc(bufW, bufA);
#else
   waveOutGetErrorText(rv, bufA, WINERR_MAX - 1);
#endif
   aud_handle = 0;
   scr_error("Can't open audio device (%d-bit stereo, %dHz):\n  %s",
	out_mode? 16 : 8, out_rate, bufA);
  }

  aud_isopen = 1;

  if(fmt.nChannels != 2)
   scr_error("Can't open audio device in stereo");
  if(fmt.wBitsPerSample != (out_mode? 16 : 8))
   scr_error("Can't open audio device in %d-bit mode", out_mode? 16 : 8);

  aud_current = 0;
  aud_cnt = 0;

  aud_head = Alloc(winBufferCount * sizeof(WAVEHDR *));

  for(a = 0; a < winBufferCount; a++)
  {
   char *p = (char *)Alloc(sizeof(WAVEHDR) + winBufferSize);
   WAVEHDR *w = aud_head[a] = (WAVEHDR *)p;

   w -> lpData = (LPSTR)p + sizeof(WAVEHDR);
   w -> dwBufferLength = (DWORD)winBufferSize;
   w -> dwBytesRecorded = 0L;
   w -> dwUser = 0;
   w -> dwFlags = 0;
   w -> dwLoops = 0;
   w -> lpNext = 0;
   w -> reserved = 0;

   rv = waveOutPrepareHeader(aud_handle, w, sizeof(WAVEHDR));
   if(rv != MMSYSERR_NOERROR)
   {
    char bufA[WINERR_MAX];
#ifdef UNICODE
    TCHAR bufW[WINERR_MAX];

    waveOutGetErrorText(rv, bufW, WINERR_MAX - 1);
    uni2asc(bufW, bufA);
#else
    waveOutGetErrorText(rv, bufA, WINERR_MAX - 1);
#endif
    scr_error("Can't setup a wave header %d:\n  %s", a, bufA);
   }
   w -> dwFlags |= WHDR_DONE;
   ++aud_whprep_cnt;
  }

  out_rate = fmt.nSamplesPerSec;
  out_bsiz = winBufferSize;
  out_blen = out_mode? out_bsiz / 2 : out_bsiz;
  out_bps = out_mode? 4 : 2;
  out_buf = (short*)Alloc(out_blen * sizeof(short));
  out_buf_lo = (int)(0x10000 * 1000.0 * 0.5 * out_blen / out_rate);
  out_buf_ms = out_buf_lo >> 16;
  out_buf_lo &= 0xFFFF;
  out_fd = (int)AU_NO_HANDLE;
  tmp_buf = (int*)Alloc(out_blen * sizeof(int));

  if(!opt_Q)
   warn("Outputting %d-bit audio at %d Hz with %d %d-sample fragments, "
	"%d ms per fragment",
	out_mode? 16 : 8, out_rate, winBufferCount, out_blen / 2, out_buf_ms);
 }	// WIN_AUDIO
#endif

#ifdef MAC_AUDIO
 // Mac CoreAudio for OS X
 {
  char deviceName[256];
  OSStatus err;
  UInt32 propertySize, bufferByteCount;
  struct AudioStreamBasicDescription streamDesc;
  int old_out_rate = out_rate;

  out_bsiz = BUFFER_SIZE;
  out_blen = out_mode? out_bsiz / 2 : out_bsiz;
  out_bps = out_mode? 4 : 2;
  out_buf = (short*)Alloc(out_blen * sizeof(short));
  out_buf_lo = (int)(0x10000 * 1000.0 * 0.5 * out_blen / out_rate);
  out_buf_ms = out_buf_lo >> 16;
  out_buf_lo &= 0xFFFF;
  tmp_buf = (int *)Alloc(out_blen * sizeof(int));

  // N.B.  Both -r and -b flags are totally ignored for CoreAudio --
  // we just use whatever the default device is set to, and feed it
  // floats.
  out_mode = 1;
  out_fd = AU_NO_HANDLE;

  // Find default device
  propertySize = sizeof(aud_dev);
  if((err= AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,
	&propertySize, &aud_dev)))
   scr_error("Get default output device failed, status = %d", (int)err);

  if(aud_dev == kAudioDeviceUnknown)
   scr_error("No default audio device found");

  // Get device name
  propertySize = sizeof(deviceName);
  if((err= AudioDeviceGetProperty(aud_dev, 1, 0,
	kAudioDevicePropertyDeviceName,
	&propertySize, deviceName)))
   scr_error("Get audio device name failed, status = %d", (int)err);

  // Get device properties
  propertySize = sizeof(streamDesc);
  if((err = AudioDeviceGetProperty(aud_dev, 1, 0,
	kAudioDevicePropertyStreamFormat,
	&propertySize, &streamDesc)))
   scr_error("Get audio device properties failed, status = %d", (int)err);

  out_rate = (int)streamDesc.mSampleRate;

  if(streamDesc.mChannelsPerFrame != 2)
   scr_error("SBaGen requires a stereo output device --\n"
	"default output has %d channels",
	streamDesc.mChannelsPerFrame);

  if(streamDesc.mFormatID != kAudioFormatLinearPCM ||
	!(streamDesc.mFormatFlags & kLinearPCMFormatFlagIsFloat))
   scr_error("Expecting a 32-bit float linear PCM output stream --\n"
	"default output uses another format");

  // Set buffer size
  bufferByteCount = BUFFER_SIZE / 2 * sizeof(float);
  propertySize = sizeof(bufferByteCount);
  if((err = AudioDeviceSetProperty(aud_dev, 0, 0, 0,
	kAudioDevicePropertyBufferSize,
	propertySize, &bufferByteCount)))
   scr_error("Set audio output buffer size failed, status = %d", (int)err);

  // Setup callback and start it
  err = AudioDeviceAddIOProc(aud_dev, mac_callback, (void *)1);
  err = AudioDeviceStart(aud_dev, mac_callback);

  // Report settings
  if(!opt_Q)
  {
   if(old_out_rate != out_rate && !out_rate_def)
    warn("*** WARNING: Non-default sampling rates not yet supported on OS X ***");
    warn("Outputting %d-bit audio at %d Hz to \"%s\",\n"
	"  using %d %d-sample fragments, %d ms per fragment",
	(int)streamDesc.mBitsPerChannel, out_rate, deviceName,
	BUFFER_COUNT, out_blen / 2, out_buf_ms);
  }
 }	// MAC_AUDIO
#endif

#ifdef NO_AUDIO
 scr_error("Direct output to soundcard not supported on this platform.\n"
	"Use -o or -O to write raw data, or -Wo or -WO to write a WAV file.");
#endif
}

#ifdef WIN_AUDIO
/* Audio callback for Win32
*/
void CALLBACK win32_audio_callback(HWAVEOUT hand, UINT uMsg,
	DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
 switch(uMsg)
 {
  case WOM_CLOSE:
   {
    static int a = 0;
    a = 1;
   }
   break;

  case WOM_OPEN:
   break;

  case WOM_DONE:
   aud_cnt--;
   //debug("Buffer done (cnt==%d)", aud_cnt);
   //debug_win32_buffer_status();
   break;
 }
}

/* type debug info about WIN32 audio buffer status (not in use now)
*/
void debug_win32_buffer_status(void)
{
 char tmp[80];
 char *p = tmp;
 int a;

 for(a = 0; a < winBufferCount; a++)
 {
  *p++ = (aud_head[a] -> dwFlags & WHDR_INQUEUE)? 'I' : '-';
  *p++ = (aud_head[a] -> dwFlags & WHDR_DONE)? 'D' : '-';
  *p++ = ' ';
 }
 p[-1]= 0;
 debug(tmp);
}
#endif

#ifdef MAC_AUDIO
/* Audio callback for Mac OS X
*/
OSStatus mac_callback(AudioDeviceID inDevice,
	const AudioTimeStamp *inNow,
	const AudioBufferList *inInputData,
	const AudioTimeStamp *inInputTime,
	AudioBufferList *outOutputData,
	const AudioTimeStamp *inOutputTime,
	void *inClientData)
{
 float *fp = outOutputData -> mBuffers[0].mData;
 int cnt = BUFFER_SIZE / 2;
 short *sp;

 if(aud_rd == aud_wr)
 {
  // Nothing in buffer list, so fill with silence
  while(cnt-- > 0)
   *fp++ = 0.0;
 }
 else
 {
  // Consume a buffer
  sp = (short *)aud_buf[aud_rd];
  while(cnt-- > 0)
   *fp++ = *sp++ * (1.0 / 32768.0);
  aud_rd = (aud_rd + 1) % BUFFER_COUNT;
 }

 return kAudioHardwareNoError;
}
#endif

/* write buffer to the audio device or output file
*/
void writeOut(char *buf, int siz)
{
 int rv;

#ifdef WIN_AUDIO
 if(out_fd == (int)AU_NO_HANDLE)
 {
  // Win32 output: write it to a header and send it off
  MMRESULT rv;

  //debug_win32_buffer_status();

  //while (aud_cnt == winBufferCount) {
  //while (aud_head[aud_current]->dwFlags & WHDR_INQUEUE) {
  while(!(aud_head[aud_current] -> dwFlags & WHDR_DONE))
  {
   //debug("SLEEP %d", out_buf_ms / 2 + 1);
   Sleep(out_buf_ms / 2 + 1);
   //debug_win32_buffer_status();
  }

  memcpy(aud_head[aud_current] -> lpData, buf, siz);
  aud_head[aud_current] -> dwBufferLength = (DWORD)siz;

  //debug("Output buffer %d", aud_current);
  rv = waveOutWrite(aud_handle, aud_head[aud_current], sizeof(WAVEHDR));

  if(rv != MMSYSERR_NOERROR)
  {
   char bufA[WINERR_MAX];
#ifdef UNICODE
   TCHAR bufW[WINERR_MAX];

   waveOutGetErrorText(rv, bufW, WINERR_MAX - 1);
   uni2asc(bufW, bufA);
#else
   waveOutGetErrorText(rv, bufA, WINERR_MAX - 1);
#endif
   scr_error("Error writing a fragment to the audio device:\n  %s", bufA);
  }

  aud_cnt++;
  aud_current++;
  aud_current %= winBufferCount;

  return;
 }
#endif

#ifdef MAC_AUDIO
 if(out_fd == AU_NO_HANDLE)
 {
  int new_wr = (aud_wr + 1) % BUFFER_COUNT;

  // Wait until there is space
  while(new_wr == aud_rd)
   delay(20);

  memcpy(aud_buf[aud_wr], buf, siz);
  aud_wr = new_wr;

  return;
 }
#endif

 // file handle != AU_NO_HANDLE => output to a file
 while(-1 != (rv = write(out_fd, buf, siz)))
 {
  if(0 == (siz -= rv))
   return;
  buf += rv;
 }
 scr_error("Output error");
}

/* cleanup audio output (handles only) (no code for MAC_AUDIO!)
*/
void audio_cleanup(void)
{
#ifdef WIN_AUDIO
 int a;

 // note, that "restart application" message is only for GUI.
 // in general, such messages is unreachable
 if(aud_isopen)
 {
  // the errors here almost impossible
  if(waveOutReset(aud_handle) != MMSYSERR_NOERROR)
   warn("\nCan't reset sound output, restart the application!");
#if DEBUG_CLEANUP
  else
   debug("-- Audio reset - ok");
#endif
  // Now we *MUST* to unprepare headers.
  for(a = 0; a < aud_whprep_cnt; a++)	// only really prepared headers!
  {
   MMRESULT rv;
   WAVEHDR *w = aud_head[a];

   if(w)
   {
    rv = waveOutUnprepareHeader(aud_handle, w, sizeof(WAVEHDR));
    if(rv != MMSYSERR_NOERROR)
    {
     char bufA[WINERR_MAX];
#ifdef UNICODE
     TCHAR bufW[WINERR_MAX];

     waveOutGetErrorText(rv, bufW, WINERR_MAX - 1);
     uni2asc(bufW, bufA);
#else
     waveOutGetErrorText(rv, bufA, WINERR_MAX - 1);
#endif
     warn("\nCan't free wave header %d:\n  %s\n"
	"Restart the application!", a, bufA);
    }
   }
  }

  // close the device as usual way
  if(waveOutClose(aud_handle) != MMSYSERR_NOERROR)
   warn("\nCan't close sound output, restart the application!");
#if DEBUG_CLEANUP
  else
   debug("-- Audio close - ok");
#endif
  aud_isopen = 0;
  aud_handle = 0;
  // out_file can't be open at this time, but no error, if we continue checks
 }
#endif

 if(out_file != NULL)
 {
  if(out_file != stdout)
  {
   fclose(out_file);
#if DEBUG_CLEANUP
   debug("-- File output close -- ok");
#endif
  }
  else
   scr_fflush(out_file);	// this unreachable for the GUI
  out_file = NULL;
  out_fd = -1;
  return;
 }

#ifdef OSS_AUDIO
 if(out_fd >= 0)
 {
  close(out_fd);
#if DEBUG_CLEANUP
  debug("-- OSS device close -- ok");
#endif
  out_fd = -1;
 }
#endif
}

/* The END...
*/

