/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen Data Computation routines

#include "sbagenr.h"

/* the sine table creation
*/
void init_sin_table(void)
{
 int a;
 int *arr = (int*)Alloc(ST_SIZ * sizeof(int));

 for(a = 0; a < ST_SIZ; a++)
  arr[a] = (int)(ST_AMP * sin((a * 3.14159265358979323846 * 2) / ST_SIZ));

 sin_table = arr;
}

/* Output a chunk of sound (a buffer-ful), then return
 *
 * Note: Optimised for 16-bit output.  Eight-bit output is
 * slower, but then it probably won't have to run at as high a
 * sample rate.
 */
void outChunk(void)
{
 static int rand0 = 0, rand1 = 0;
 int off = 0;

 if(mix_in)
 {
  int rv = inbuf_read(tmp_buf, out_blen);

  if(rv == 0)
  {
   if(!opt_Q)
    warn("\nEnd of mix input audio stream");
   exitSBaGen(0);
  }

  while(rv < out_blen)
   tmp_buf[rv++] = 0;
 }
   
 while (off < out_blen)
 {
  int ns = noise2();		// Use same pink noise source for everything
  int tot1, tot2;		// Left and right channels
  int mix1, mix2;		// Incoming mix signals
  int val, a;
  Channel *ch;
  int *tab;

  mix1 = tmp_buf[off];
  mix2 = tmp_buf[off + 1];

  // Do default mixing at 100% if no mix/* stuff is present
  if(!mix_flag)
  {
   tot1 = mix1 << 12;
   tot2 = mix2 << 12;
  }
  else
  {
   tot1 = tot2 = 0;
  }

// mute flag check (Win32 GUI only)
#ifdef SBG_GUI
#define QMUTE	     if(!isChMute[a])
#else
#define QMUTE
#endif

  ch = &chan[0];
  for(a = 0; a < N_CH; a++, ch++)
   switch(ch -> typ)
   {
    case 0:
     break;

    case 1:	// Binaural tones
     ch -> off1 += ch -> inc1;
     ch -> off1 &= (ST_SIZ << 16) - 1;
     QMUTE
      tot1 += ch -> amp * sin_table[ch -> off1 >> 16];
     ch -> off2 += ch -> inc2;
     ch -> off2 &= (ST_SIZ << 16) - 1;
     QMUTE
      tot2 += ch -> amp2 * sin_table[ch -> off2 >> 16];
     break;

    case 2:	// Pink noise
     val = ns * ch -> amp;
     QMUTE
     {
      tot1 += val;
      tot2 += val;
     }
     break;

    case 3:	// Bell
     if(ch -> off2)
     {
      ch -> off1 += ch -> inc1;
      ch -> off1 &= (ST_SIZ << 16) - 1;
      val = ch -> off2 * sin_table[ch -> off1 >> 16];
      QMUTE
      {
       tot1 += val;
       tot2 += val;
      }
      if(--ch -> inc2 < 0)
      {
       ch -> inc2 = out_rate / 20;
       ch -> off2 -= 1 + ch -> off2 / 12;	// Knock off 10% each 50 ms
      }
     }
     break;

    case 4:	// Spinning pink noise
     ch -> off1 += ch -> inc1;
     ch -> off1 &= (ST_SIZ << 16) - 1;
     val = (ch -> inc2 * sin_table[ch -> off1 >> 16]) >> 24;
     QMUTE
     {
      tot1 += ch -> amp * noise_buf[(uchar)(noise_off + 128 + val)];
      tot2 += ch -> amp * noise_buf[(uchar)(noise_off + 128 - val)];
     }
     break;

    case 5:	// Mix level
     QMUTE
     {
      tot1 += mix1 * ch -> amp;
      tot2 += mix2 * ch -> amp;
     }
     break;

    default:	// Waveform-based binaural tones
     tab = waves[-1 - ch -> typ];
     ch -> off1 += ch -> inc1;
     ch -> off1 &= (ST_SIZ << 16) - 1;
     QMUTE
      tot1 += ch -> amp * tab[ch -> off1 >> 16];
     ch -> off2 += ch -> inc2;
     ch -> off2 &= (ST_SIZ << 16) - 1;
     QMUTE
      tot2 += ch -> amp * tab[ch -> off2 >> 16];
     break;
   }		// switch() / for(..a..)

#undef QMUTE

  // // Add pink noise as dithering
  // tot1 += (ns >> NS_DITHER) + 0x8000;
  // tot2 += (ns >> NS_DITHER) + 0x8000;
      
  // // Add white noise as dithering
  // tot1 += (seed >> 1) + 0x8000;
  // tot2 += (seed >> 1) + 0x8000;

  // White noise dither; you could also try (rand0-rand1) for a
  // dither with more high frequencies
  rand0 = rand1; 
  rand1 = (rand0 * 0x660D + 0xF35F) & 0xFFFF;
  if(tot1 <= 0x7FFF0000)
   tot1 += rand0;
  if(tot2 <= 0x7FFF0000)
   tot2 += rand0;

  out_buf[off++] = tot1 >> 16;
  out_buf[off++] = tot2 >> 16;
 }		// while(..buflen..)

 // Generate debugging amplitude output
 if(DEBUG_DUMP_AMP)
 {
  short *sp = out_buf;
  short *end = out_buf + out_blen;
  int max = 0;

  while(sp < end)
  {
   int val = (int)sp[0] + (int)sp[1];

   sp += 2;
   if(val < 0)
    val = -val;
   if(val > max)
    max = val;
  }
  max /= 328;
  while(max-- > 0)
   scr_putc('#', stdout);
  scr_putc('\n', stdout);
  scr_fflush(stdout);
 }

 // Rewrite buffer for 8-bit mode
 if(out_mode == 0)
 {
  short *sp = out_buf;
  short *end = out_buf + out_blen;
  char *cp = (char*)out_buf;

  while(sp < end)
   *cp++ = (*sp++ >> 8) + 128;
 }

 // Rewrite buffer for 16-bit byte-swapping
 if(out_mode == 2)
 {
  char *cp= (char*)out_buf;
  char *end= (char*)(out_buf + out_blen);

  while(cp < end)
  {
   char tmp = *cp++;
   cp[-1] = cp[0];
   *cp++ = tmp;
  }
 }

 // Check and update the byte count if necessary
 if(byte_count > 0)
 {
  if(byte_count <= out_bsiz)
  {
   writeOut((char*)out_buf, (int)byte_count);	// safe conversion
   exitSBaGen(0);	// All done
  }
  else
  {
   writeOut((char*)out_buf, out_bsiz);
   byte_count -= out_bsiz;
  }
 }
 else
  writeOut((char*)out_buf, out_bsiz);
}

/* Calculate amplitude adjustment factor for frequency 'freq'
*/
double ampAdjust(double freq)
{
 int a;
 AmpAdj *p0, *p1;

 if(!opt_c)
  return 1.0;
 if(freq <= ampadj[0].freq)
  return ampadj[0].adj;
 if(freq >= ampadj[opt_c - 1].freq)
  return ampadj[opt_c - 1].adj;

 for(a = 1; a < opt_c; a++)
  if(freq < ampadj[a].freq) 
   break;
   
 p0 = &ampadj[a - 1];
 p1 = &ampadj[a];
      
 return p0 -> adj + (p1 -> adj - p0 -> adj) *
	(freq - p0 -> freq) / (p1 -> freq - p0 -> freq);
}
   
/* Correct channel values and types according to current period,
 * and current time
 */
void corrVal(int running)
{
 int a;
 int t0 = per -> tim;
 int t1 = per -> nxt -> tim;
 Channel *ch;
 Voice *v0, *v1, *vv;
 double rat0, rat1;
 int trigger = 0;

 // Move to the correct period
 while((now >= t0) ^ (now >= t1) ^ (t1 > t0))
 {
  per = per -> nxt;
  t0 = per -> tim;
  t1 = per -> nxt -> tim;
  if(running)
  {
   if(tty_erase)
   {
#ifdef ANSI_TTY
    scr_fprintf(stderr, "\033[K");
#else
#ifndef SBG_GUI
    scr_fprintf(stderr, "%*s\r", tty_erase, "");
#endif
    tty_erase = 0;
#endif
   }
   dispCurrPer(stderr);
   status(0);
  }
  trigger = 1;		// Trigger bells or whatever
 }

 // Run through to calculate voice settings for current time
 rat1 = (double)t_per0(t0, now) / (double)t_per24(t0, t1);
 rat0 = 1.0 - rat1;
 for(a = 0; a < N_CH; a++)
 {
  ch = &chan[a];
  v0 = &per -> v0[a];
  v1 = &per -> v1[a];
  vv = &ch -> v;

  if(vv -> typ != v0 -> typ)
  {
   switch(vv -> typ = ch -> typ = v0 -> typ)
   {
    case 1:
     ch -> off1 = ch -> off2 = 0;
     break;

    case 2:
     break;

    case 3:
     ch -> off1 = ch -> off2 = 0;
     break;

    case 4:
     ch -> off1 = ch -> off2 = 0;
     break;

    case 5:
     break;

    default:
     ch -> off1 = ch -> off2 = 0;
     break;
   }
  }
      
  // Setup vv->*
  switch(vv -> typ)
  {
   case 1:
    vv -> amp  = rat0 * v0 -> amp  + rat1 * v1 -> amp;
    vv -> carr = rat0 * v0 -> carr + rat1 * v1 -> carr;
    vv -> res  = rat0 * v0 -> res  + rat1 * v1 -> res;
    break;

   case 2:
    vv -> amp = rat0 * v0 -> amp + rat1 * v1 -> amp;
    break;

   case 3:
    vv -> amp  = v0 -> amp;	// No need to slide, as bell only rings briefly
    vv -> carr = v0 -> carr;
    break;

   case 4:
    vv -> amp  = rat0 * v0 -> amp  + rat1 * v1 -> amp;
    vv -> carr = rat0 * v0 -> carr + rat1 * v1 -> carr;
    vv -> res  = rat0 * v0 -> res  + rat1 * v1 -> res;
    if(vv -> carr > spin_carr_max) // Clipping sweep width
     vv -> carr = spin_carr_max;
    if(vv -> carr < -spin_carr_max)
     vv -> carr = -spin_carr_max;
    break;

   case 5:
    vv -> amp = rat0 * v0 -> amp + rat1 * v1 -> amp;
    break;

   default:			// Waveform based binaural
    vv -> amp  = rat0 * v0 -> amp  + rat1 * v1 -> amp;
    vv -> carr = rat0 * v0 -> carr + rat1 * v1 -> carr;
    vv -> res  = rat0 * v0 -> res  + rat1 * v1 -> res;
    break;
  }
 }

 // Check and limit amplitudes if -c option in use
 if(opt_c)
 {
  double tot_beat = 0, tot_other = 0;

  for(a = 0; a < N_CH; a++)
  {
   vv = &chan[a].v;
   if(vv -> typ == 1)
   {
    double adj1 = ampAdjust(vv -> carr + vv -> res / 2);
    double adj2 = ampAdjust(vv -> carr - vv -> res / 2);

    if(adj2 > adj1) adj1 = adj2;
    tot_beat += vv -> amp * adj1;
   }
   else
    if(vv->typ)
    {
     tot_other += vv -> amp;
    }
  }		// for(..a..)

  if(tot_beat + tot_other > 4096)
  {
   double adj_beat = (tot_beat > 4096)? 4096 / tot_beat : 1.0;
   double adj_other = (4096 - tot_beat * adj_beat) / tot_other;

   for(a = 0; a < N_CH; a++)
   {
    vv = &chan[a].v;
    if(vv -> typ == 1)
     vv -> amp *= adj_beat;
    else
     if(vv -> typ)
      vv->amp *= adj_other;
   }		// for(..a..)
  }
 }		// if(opt_c)
   
 // Setup Channel data from Voice data
 for(a = 0; a < N_CH; a++)
 {
  ch = &chan[a];
  vv = &(ch -> v);
      
  // Setup ch->* from vv->*
  switch(vv -> typ)
  {
   double freq1, freq2;

   case 1:
    freq1 = vv -> carr + vv -> res / 2;
    freq2 = vv -> carr - vv -> res / 2;
    if(opt_c)
    {
     ch -> amp  = (int)(vv -> amp * ampAdjust(freq1));
     ch -> amp2 = (int)(vv -> amp * ampAdjust(freq2));
    }
    else
     ch -> amp = ch -> amp2 = (int)(vv -> amp);

    ch -> inc1 = (int)(freq1 / out_rate * ST_SIZ * 65536);
    ch -> inc2 = (int)(freq2 / out_rate * ST_SIZ * 65536);
    break;

   case 2:
    ch -> amp = (int)(vv -> amp);
    break;

   case 3:
    ch -> amp  = (int)(vv -> amp);
    ch -> inc1 = (int)(vv -> carr / out_rate * ST_SIZ * 65536);
    if(trigger)		// Trigger the bell only on entering the period
    {
     ch -> off2 = ch -> amp;
     ch -> inc2 = out_rate / 20;
    }
    break;

   case 4:
    ch -> amp= (int)(vv -> amp);
    ch -> inc1 = (int)(vv -> res / out_rate * ST_SIZ * 65536);
    ch -> inc2 = (int)(vv -> carr * 1.0E-6 * out_rate * (1 << 24) / ST_AMP);
    break;

   case 5:
    ch -> amp = (int)(vv -> amp);
    break;

   default:		// Waveform based binaural
    ch -> amp = (int)(vv -> amp);
    ch -> inc1 = (int)((vv -> carr + vv -> res / 2) / out_rate * ST_SIZ * 65536);
    ch -> inc2 = (int)((vv -> carr - vv -> res / 2) / out_rate * ST_SIZ * 65536);
    if(ch -> inc1 > ch -> inc2)
     ch -> inc2 = -(ch -> inc2);
    else
     ch -> inc1 = -(ch -> inc1);
    break;
  }		// switch(...)
 }		// for(..a..)
}

/* Takes a set of points and repeats them twice, inverting the
 * second set, and then interpolates them using a periodic sinc
 * function (see http://www-ccrma.stanford.edu/~jos/resample/)
 * and writes them to arr[] in the same format as the sin_table[].
 */
void sinc_interpolate(double *dp, int np, int *arr)
{
 double *sinc;		// Temporary sinc-table
 double *out;		// Temporary output table
 int a, b;
 double dmax, dmin;
 double adj, off;

 // Generate a modified periodic sin(x)/x function to be used for
 // each of the points.  Really this should be sin(x)/x modified
 // by the sum of an endless series.  However, this doesn't
 // converge very quickly, so to save time I'm approximating this
 // series by 1-4*t*t where t ranges from 0 to 0.5 over the first
 // half of the periodic cycle.  If you do the maths, this is at
 // most 5% out.  This will have to do - it's smooth, and I don't
 // know enough maths to make this series converge quicker.
 sinc = (double *)Alloc(ST_SIZ * sizeof(double));
 sinc[0] = 1.0;
 for(a = ST_SIZ / 2; a > 0; a--)
 {
  double tt  = a * 1.0 / ST_SIZ;
  double t2  = tt * tt;
  double adj = 1 - 4 * t2;
  double xx  = 2 * np * 3.14159265358979323846 * tt;
  double vv  = adj * sin(xx) / xx;

  sinc[a] = vv;
  sinc[ST_SIZ - a] = vv;
 }

 // Build waveform into buffer
 out = (double *)Alloc(ST_SIZ * sizeof(double));
 for(b = 0; b < np; b++)
 {
  int off = b * ST_SIZ / np / 2;
  double val = dp[b];

  for(a = 0; a < ST_SIZ; a++)
  {
   out[(a + off) & (ST_SIZ - 1)] += sinc[a] * val;
   out[(a + off + ST_SIZ / 2) & (ST_SIZ - 1)] -= sinc[a] * val;
  }
 }

 // Look for maximum for normalization
 dmax = dmin = 0;
 for(a = 0; a < ST_SIZ; a++)
 {
  if(out[a] > dmax)
   dmax = out[a];
  if(out[a] < dmin)
   dmin = out[a];
 }

 // Write out to output buffer
 off = -0.5 * (dmax + dmin);
 adj = ST_AMP / ((dmax - dmin) / 2);
 for(a = 0; a < ST_SIZ; a++)
  arr[a] = (int)((out[a] + off) * adj);

 MFree(sinc);
 MFree(out);
}

/* The END...
*/

