/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen Console I/O routines

#include "sbagenr.h"

/* Update a status line
*/
void status(char *err)
{
 int a;
 int nch = N_CH;
 char *p = buf, *p0, *p1;

 if(opt_Q)
  return;

#ifdef ANSI_TTY
 if (tty_erase)
  p += sprintf(p, "\033[K");
#endif

 p0 = p;		// Start of line
 *p++ = ' ';
 *p++ = ' ';
 p += sprintTime(p, now);
 while(nch > 1 && chan[nch-1].v.typ == 0)
  nch--;
 for(a= 0; a<nch; a++)
  p += sprintVoice(p, &chan[a].v, 0);
 if(err)
  p += sprintf(p, " %s", err);
 p1 = p;		// End of line

#ifndef ANSI_TTY
#ifndef SBG_GUI
 while(tty_erase > p - p0)
  *p++ = ' ';
#endif
#endif

 tty_erase = p1 - p0;		// Characters that will need erasing
 scr_fprintf(stderr, "%s\r", buf);
 scr_fflush(stderr);
}

/* Display current period details
*/
void dispCurrPer(FILE *fp)
{
 int a;
 Voice *v0, *v1;
 char *p0, *p1;
 int len0, len1;
 int nch = N_CH;

 if(opt_Q)
  return;

 p0 = buf;
 p1 = buf_copy;

 p0 += sprintf(p0, "* ");
 p0 += sprintTime(p0, per -> tim);
 p1 += sprintf(p1, "  ");
 p1 += sprintTime(p1, per -> nxt -> tim);

 v0 = per -> v0;
 v1 = per -> v1;
 while(nch > 1 && v0[nch - 1].typ == 0)
  nch--;
 for(a = 0; a < nch; a++, v0++, v1++)
 {
  p0 += len0 = sprintVoice(p0, v0, 0);
  p1 += len1 = sprintVoice(p1, v1, v0);
  while(len0 < len1)
  {
   *p0++= ' ';
   len0++;
  }
  while(len1 < len0)
  {
   *p1++= ' ';
   len1++;
  }
 }
 *p0 = 0;
 *p1 = 0;
 scr_fprintf(fp, "%s\n%s\n", buf, buf_copy);
 scr_fflush(fp);
}

#ifndef	SBG_GUI
/* print error message and terminate
*/
void error(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 // plain command line
 vfprintf(stderr, fmt, ap);
 fprintf(stderr, "\n");
#ifdef EXIT_KEY
 fprintf(stderr, "Press <RETURN> to continue: ");
 fflush(stderr);
 while(getchar() != '\n') /* wait the '\n' */ ;
#endif
 exitSBaGen(1);
}
#endif

/* print debug message
*/
void debug(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 scr_vfprintf(stderr, fmt, ap);
 scr_putc('\n', stderr);
}

/* print warning message
*/
void warn(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 scr_vfprintf(stderr, fmt, ap);
 scr_putc('\n', stderr);
}

/* The END...
*/

