/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen Info output file

#include "sbagenr.h"

/* print help and terminate
*/
void help(void)
{
 scr_printf("SBaGen - Sequenced Binaural Beat Generator, version " VERSION 
	NL "Copyright (c) 1999-2004 Jim Peters, http://uazu.net/, all rights "
	NL "  reserved, released under the GNU GPL v2.  See file COPYING."
	NL "Remastered by Unknown corp. under the GNU GPL v2, Internal usage only"
	NL 
	NL "Usage: sbagen [options] seq-file ..."
	NL "       sbagen [options] -i tone-specs ..."
	NL "       sbagen [options] -p pre-programmed-sequence-specs ..."
	NL
	NL "Options:  -h        Display this help-text"
	NL "          -Q        Quiet - don't display running status"
	NL "          -D        Display the full interpreted sequence instead of playing it"
	NL "          -i        Immediate.  Take the remainder of the command line to be"
	NL "                     tone-specifications, and play them continuously"
	NL "          -p        Pre-programmed sequence.  Take the remainder of the command"
	NL "                     line to be a type and arguments, e.g. \"drop 00ds+\""
	NL "          -q mult   Quick.  Run through quickly (real time x 'mult') from the"
	NL "                     start time, rather than wait for real time to pass"
	NL
	NL "          -r rate   Select the output rate (default is 44100 Hz, or from -m)"
#ifndef MAC_AUDIO
	NL "          -b bits   Select the number bits for output (8 or 16, default 16)"
#endif
#ifdef	WIN_AUDIO
	NL "          -B bSize,bCnt Set the size (>=1024) and count (>=2) of Windows"
	NL "                    audio buffers (default -B 32768,8) (all parameters"
	NL "                    will be rounded down to power of two)"
#endif
	NL "          -L time   Select the length of time (hh:mm or hh:mm:ss) to output"
	NL "                     for.  Default is to output forever."
	NL "          -S        Output from the first tone-set in the sequence (Start),"
	NL "                     instead of working in real-time.  Equivalent to '-q 1'."
	NL "          -E        Output until the last tone-set in the sequence (End),"
	NL "                     instead of outputting forever."
	NL "          -T time   Start at the given clock-time (hh:mm)"
	NL
	NL "          -o file   Output raw data to the given file instead of /dev/dsp"
#ifndef SBG_GUI
	NL "          -O        Output raw data to the standard output"
#endif
	NL "          -W        Output a WAV-format file instead of raw data"
	NL "          -m file   Read audio data from the given file and mix it with the"
	NL "                      generated binaural beats; may be "
#ifdef OGG_DECODE
	"ogg/"
#endif
#ifdef MP3_DECODE
	"mp3/"
#endif
	"wav/raw format"
	NL "          -m+ file  Same as -m, but wav/raw mix play in endless loop"
#ifdef	OGG_DECODE
	NL "          -l numb   Alternative way to set <numb> for file.ogg#numb"
#endif
#ifndef SBG_GUI
	NL "          -M        Read raw audio data from the standard input and mix it"
	NL "                      with the generated binaural beats (raw only)"
#endif
	NL
	NL "          -R rate   Select rate in Hz that frequency changes are recalculated"
	NL "                     (for file/pipe output only, default is 10Hz)"
	NL "          -F fms    Fade in/out time in ms (default 60000ms, or 1min)"
#ifdef OSS_AUDIO
	NL "          -d dev    Select a different output device instead of /dev/dsp"
#endif
	NL "          -c spec   Compensate for low-frequency headphone roll-off; see docs"
	NL);
 exitSBaGen(0);
}

/* print usage information and terminate
*/
void usage(void)
{
 scr_error("SBaGen - Sequenced Binaural Beat Generator, version " VERSION 
	NL "Copyright (c) 1999-2004 Jim Peters, http://uazu.net/, all rights "
	NL "  reserved, released under the GNU GPL v2.  See file COPYING."
	NL "Remastered by Unknown corp. under the GNU GPL v2, Internal usage only"
	NL 
	NL "Usage: sbagen [options] seq-file ..."
	NL "       sbagen [options] -i tone-specs ..."
	NL "       sbagen [options] -p pre-programmed-sequence-specs ..."
	NL
	NL "For full usage help, type 'sbagen -h'.  For latest version see"
	NL "http://uazu.net/sbagen/ or http://sbagen.sf.net/"
#ifdef EXIT_KEY
	NL
	NL "Windows users please note that this utility is designed to be run as the"
	NL "associated application for SBG files.  This should have been set up for you by"
	NL "the installer.  You can run all the SBG files directly from the desktop by"
	NL "double-clicking on them, and edit them using NotePad from the right-click menu."
	NL "Alternatively, SBaGen may be run from the MS-DOS prompt (CMD on WinXP), or from"
	NL "BAT files.  SBaGen is powerful software -- it is worth the effort of figuring"
	NL "all this out.  See SBAGEN.TXT for the full documentation."
	NL
	NL "Editing the SBG files gives you access to the full tweakable power of SBaGen,"
	NL "but if you want a simple GUI interface to the most basic features, you could "
	NL "look at a user-contributed tool called SBaGUI:"
	NL
	NL "  http://sbagen.opensrc.org/wiki.php?page=SBaGUI"
#endif
	NL);
}

/* Error for bad p-drop args
*/
void bad_drop(void)
{
 scr_error("Bad arguments: expecting -p drop [<time-spec>] <drop-spec> [<tone-specs...>]"
	NL "<drop-spec> is <digit><digit>[.<digit>...]<a-l>[s|k][+][^|!][/<amp>]"
	NL "(the optional wake-up symbols '^' and '!' are equivalent for this version)"
	NL "The optional <time-spec> is t<drop-time>,<hold-time>,<wake-time>, all times"
	NL "  in minutes (the default is equivalent to 't30,30,3')."
	NL "The optional <tone-specs...> let you mix other stuff with the drop"
	NL "  sequence like pink noise or a mix soundtrack, e.g 'pink/20' or 'mix/60'");
}

/* terminate, when bad slide found
*/
void bad_slide(void)
{
 scr_error("Bad arguments: expecting -p slide [<time-spec>] <slide-spec> [<tone-specs...>]"
	NL "<slide-spec> is just like a tone-spec: <carrier><sign><beat>/<amp>"
	NL "The optional <time-spec> is t<slide-time>, giving length of session in"
	NL "  minutes (the default is equivalent to 't30')."
	NL "The optional <tone-specs...> let you mix other stuff with the drop"
	NL "  sequence like pink noise or a mix soundtrack, e.g 'pink/20' or 'mix/60'");
}

/* terminate, when bad sequence file
*/
void badSeq(void)
{
 scr_error("Bad sequence file content at line: %d\n  %s", in_lin, lin_copy);
}

/* report about bad time line and terminate
*/
void badTime(char *tim)
{
 scr_error("Badly constructed time \"%s\", line %d:\n  %s", tim, in_lin, lin_copy);
}

/* The END...
*/

