/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen -- Miscellaneous

#include "sbagenr.h"

/* initialize memory allocator
*/
void InitAlloc(void)
{
#if	DEBUG_MEM
 char mlog_name[256];
 static unsigned mlog_no = 0;
#endif

 heapRoot = heapPtr = NULL;
#if	DEBUG_MEM
 sprintf(mlog_name, "alloc%08X.log", mlog_no++);
 dmem = fopen(mlog_name, "w");
 if(dmem == NULL)
  scr_error("can't create debug memory info...");
#endif
}

/* allocate & clear memory with check (NON REENTRANT)
*/
void *Alloc(size_t len)
{
 HeapList *hl = calloc(sizeof(HeapList), 1);
 void *p = calloc(1, len);

 if(hl == NULL || p == NULL)
 {
  if(hl)
   free(hl);
  if(p)
   free(p);
  scr_error("Out of memory");
 }

 // add memory block to my list
 if(heapRoot == NULL)
 {
  heapRoot = heapPtr = hl;
  hl -> nxt = hl -> prv = NULL;
  hl -> memptr = p;
 }
 else
 {
  hl -> nxt = NULL;
  hl -> prv = heapPtr;
  hl -> memptr = p;
  heapPtr -> nxt = hl;
  heapPtr = hl;
 }

#if DEBUG_MEM
 fprintf(dmem, "-- Allc:: %08lX\n", (long)p);
#endif

 return p;
}

/* free memory, which allocated via Alloc (NON REENTRANT)
*/
void MFree(void *p)
{
 HeapList *hl;

 for(hl = heapRoot; hl; hl = hl -> nxt)
 {
  if(hl -> memptr == p)
  {
   // from end?
   if(hl -> nxt == NULL)
   {
    if(hl -> prv)
    {
     hl -> prv -> nxt = NULL;
     heapPtr = hl -> prv;
    }
    else	// list became empty
    {
     heapRoot = heapPtr = NULL;
    }

#if DEBUG_MEM
    fprintf(dmem, "-- Free:: %08lX\n", (long)p);
#endif

    free(hl);
    free(p);

    return;
   }

   // from middle?
   if(hl -> prv)	// hl -> nxt != NULL
   {
    hl -> nxt -> prv = hl -> prv;
    hl -> prv -> nxt = hl -> nxt;
   }
   else			// hl -> prv == NULL && hl -> nxt != NULL
   {			// from begin
    hl -> nxt -> prv = NULL;
    heapRoot = hl -> nxt;
   }

#if DEBUG_MEM
   fprintf(dmem, "-- Free:: %08lX\n", (long)p);
#endif

   free(hl);
   free(p);

   return;
  }
 }
 // must be unreachable
 scr_error("Invalid MFree usage");
}

/* free all the memory, allocated via Alloc
*/
void freeAll(void)
{
 HeapList *hl = heapPtr, *tmp;

 while(hl)
 {
  tmp = hl;
  hl = hl -> prv;
  // we don't need to correct link fields here

#if DEBUG_MEM
  fprintf(dmem, "-- Kill:: %08lX\n", (long)(tmp -> memptr));
#endif

  free(tmp -> memptr);
  free(tmp);
 }

#if DEBUG_MEM
 if(dmem)
 {
  fclose(dmem);
  dmem = NULL;
 }
#endif

 heapRoot = heapPtr = NULL;
}

/* duplicate string with check
*/
char *StrDup(char *str)
{
 char *rv = (char *)Alloc(strlen(str) + 1);

 if(!rv)
  scr_error("Out of memory");
 strcpy(rv, str);
 return rv;
}

/* The END...
*/

