/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is the options / script parsers

#include "sbagenr.h"


/* Scan options.  Returns a flag indicating what is expected to
 * interpret the rest of the arguments: 0 normal, 'i' immediate
 * (-i option), 'p' -p option.
 */
int scanOptions(int *acp, char ***avp)
{
 int argc = *acp;
 char **argv = *avp;
 int val;
 char dmy;
 int rv = 0;

 // Scan options
 while(argc > 0 && argv[0][0] == '-' && argv[0][1])
 {
  char opt, *p = 1 + *argv++;

  argc--;
  while((opt= *p++))
  {
   // Check options that are available on both
   switch(opt)
   {
    case 'Q':
     opt_Q = 1;
     break;

    case 'E':
     opt_E = 1;
     break;

    case 'm':
    if(argc-- < 1)
     scr_error("-m option expects filename");
     // Earliest takes precedence, so command-line overrides sequence file
     if (!opt_m)
     {
      opt_m = *argv++;
      if('+' == *p)		// -m+ -- repeat the mix file (wav/raw only)
      {
       ++p;
       opt_mrep = 1;
      }
      else
       opt_mrep = 0;
     }
     break;

    case 'S':
     opt_S = 1;
     if(!fast_mult)
      fast_mult = 1; 		// Don't try to sync with real time
     break;

    case 'L':
     if(argc-- < 1 || 0 == (val = readTime(*argv, &opt_L)) ||
	1 == sscanf(*argv++ + val, " %c", &dmy))
      scr_error("-L expects hh:mm or hh:mm:ss time");
     break;

    case 'T':
     if(argc-- < 1 || 0 == (val = readTime(*argv, &opt_T)) ||
	1 == sscanf(*argv++ + val, " %c", &dmy))
      scr_error("-T expects hh:mm or hh:mm:ss time");
     if(!fast_mult)
      fast_mult = 1;		// Don't try to sync with real time
     break;

    case 'F':
     if(argc-- < 1 || 1 != sscanf(*argv++, "%d %c", &fade_int, &dmy))
      scr_error("-F expects fade-time in ms");
    break;

    case 'c':
     if(argc-- < 1)
      scr_error("-c expects argument");
      setupOptC(*argv++);
     break;

    case 'i':
     rv = 'i';
     break;

    case 'p':
     rv = 'p';
     break;

    case 'h':
     help();
     break;

    case 'D':
     opt_D = 1;
     break;

#ifndef SBG_GUI			// -M and -O not supported for GUI
    case 'M':
     opt_M = 1;
     break;

    case 'O':
     opt_O = 1;
     if(!fast_mult)
      fast_mult = 1; 		// Don't try to sync with real time
     break;
#endif

    case 'W':
     opt_W = 1;
     if(!fast_mult)
      fast_mult = 1; 		// Don't try to sync with real time
     break;

    case 'q': 
     opt_S = 1;
     if(argc-- < 1 || 1 != sscanf(*argv++, "%d %c", &fast_mult, &dmy))
      scr_error("Expecting an integer after -q");
     if(fast_mult < 1)
      fast_mult = 1;
     break;

    case 'r':
     if(argc-- < 1 || 1 != sscanf(*argv++, "%d %c", &out_rate, &dmy))
      scr_error("Expecting an integer after -r");
     out_rate_def = 0;
     break;

#ifndef MAC_AUDIO
    case 'b':
     if(argc-- < 1 ||
	1 != sscanf(*argv++, "%d %c", &val, &dmy) ||
	!(val == 8 || val == 16))
      scr_error("Expecting -b 8 or -b 16");
     out_mode = (val == 8)? 0 : 1;
     break;
#endif

    case 'o':
     if(argc-- < 1)
      scr_error("Expecting filename after -o");
     opt_o = *argv++;
     if(!fast_mult)
      fast_mult = 1;		// Don't try to sync with real time
     break;

#ifdef OSS_AUDIO
    case 'd':
     if(argc-- < 1)
      scr_error("Expecting device filename after -d");
     opt_d = *argv++;
     break;
#endif

#ifdef WIN_AUDIO
    case 'B':			// bufferSize,bufferCount
     if(argc-- < 1 ||
	2 != sscanf(*argv++, "%u,%u %c", &winBufferSize, &winBufferCount, &dmy))
      scr_error("Expecting -B BufSize,BufCount");
     winBufferSize = dpwr2(winBufferSize);	// pwr of 2
     winBufferCount = dpwr2(winBufferCount);
     if(winBufferCount < 2 || winBufferSize < 1024)
      scr_error("Wrong value for Buffer Size (%u) or Count (%u)",
	winBufferSize, winBufferCount);
     break;

#endif

    case 'R':
     if(argc-- < 1 || 1 != sscanf(*argv++, "%d %c", &out_prate, &dmy))
      scr_error("Expecting integer after -R");
     break;

#ifdef OGG_DECODE
    case 'l':			// alternate way to set mix_cnt
     if(argc-- < 1 || 1 != sscanf(*argv++, "%u %c", &mix_cnt, &dmy))
      scr_error("Expecting an non-negative integer after -l");
     break;
#endif

    default:
     scr_error("Option -%c not known; run 'sbagen -h' for help", opt);
   }
  }
 }

 *acp = argc;
 *avp = argv;
 return rv;
}

/* Handle an option string, breaking it into an (argc/argv) list
 * for scanOptions.
 */
void handleOptions(char *str0)
{
 // Always StrDup() string and don't bother to free(), as normal
 // argv[] strings stick around for the life of the program
 char *str = StrDup(str0);
 char *argv[MAX_ARGC+1];
 int argc = 0;

 while(*str)
 {
  if(argc >= MAX_ARGC)
   scr_error("Too many options at line: %d\n  %s", in_lin, lin_copy);
  argv[argc++] = str;
  while(*str && !isspace(*str))
   str++;
  if(!*str)
   continue;
  *str++= 0;		// NUL-term this word
  while (isspace(*str))
   str++;
 }
 argv[argc] = 0;	// Terminate argv list with a NULL

 // Process the options
 {
  char **av = argv;
  int ac = argc;
  int rv;

  rv = scanOptions(&ac, &av);

  if(rv == 'i')
  {
   // Immediate mode
   readSeqImm(ac, av);
  }
  else
   if(rv == 'p')
   {
    // Pre-programmed sequence
    readPreProg(ac, av);
   }
   else
    if(ac)
     scr_error("Trailing garbage after options at line: %d\n  %s", in_lin, lin_copy);
 }
}

/* Setup the ampadj[] array from the given -c spec-string
*/
void setupOptC(char *spec)
{
 char *p = spec, *q;
 int a, b;
   
 for(;;)
 {
  while(isspace(*p) || *p == ',')
   p++;
  if(!*p)
   break;

  if(opt_c >= AMP_ADJ_SIZ /* sizeof(ampadj) / sizeof(ampadj[0]) */)
   scr_error("Too many -c option frequencies; maxmimum is %d", 
	AMP_ADJ_SIZ /* sizeof(ampadj) / sizeof(ampadj[0]) */);

  ampadj[opt_c].freq = strtod(p, &q);
  if(p == q)
   goto bad;
  if(*q++ != '=')
   goto bad;
  ampadj[opt_c].adj = strtod(q, &p);
  if(p == q)
   goto bad;
  opt_c++;
 }

 // Sort the list
 for(a = 0; a < opt_c; a++)
  for(b = a + 1; b < opt_c; b++)
   if(ampadj[a].freq > ampadj[b].freq)
   {
    double tmp;

    tmp = ampadj[a].freq;
    ampadj[a].freq = ampadj[b].freq;
    ampadj[b].freq = tmp;

    tmp= ampadj[a].adj;
    ampadj[a].adj = ampadj[b].adj;
    ampadj[b].adj = tmp;
   }
 return;		// OK
      
bad:
 scr_error("Bad -c option spec; expecting <freq>=<amp>[,<freq>=<amp>]...:\n  %s", spec);
}

/* Read a line, discarding blank lines and comments.  Rets:
 * Another line?  Comments starting with '##' are displayed on
 * stderr.
 */
int readLine(void)
{
 char *p;

 lin = buf;

 for(;;)
 {
  if(!fgets(lin, LBUF_SIZE * sizeof(buf[0]), in))
  {
   if(feof(in))
    return 0;
   scr_error("Read error on sequence file");
  }

  in_lin++;

  while(isspace(*lin))
   lin++;
  p = strchr(lin, '#');
  if(p && p[1] == '#')
   scr_fprintf(stderr, "%s", p);
  p = p? p : strchr(lin, 0);
  while(p > lin && isspace(p[-1]))
   p--;
  if(p != lin)
   break;
 }
 *p = 0;
 lin_copy = buf_copy;
 strcpy(lin_copy, lin);
 return 1;
}

/* Get next word at '*lin', moving lin onwards, or return 0
*/
char *getWord(void)
{
 char *rv, *end;

 while(isspace(*lin))
  lin++;

 if(!*lin)
  return 0;

 rv = lin;

 while(*lin && !isspace(*lin))
  lin++;

 end = lin;
 if(*lin)
  lin++;
 *end = 0;

 return rv;
}

// Convenience for creating sequences on the fly
// ----------- --- -------- --------- -- --- ---

/* Helper for 'Read a time-line of either type' --
 * -- Convenience for situations where buffer is being filled by
 * something other than readLine()
 */
void readTimeLine2(void)
{
 lin = buf;
 lin_copy = buf_copy;
 strcpy(lin_copy, lin);
 readTimeLine();
}

/* Helper for 'Read a name definition' --
 * -- Convenience for situations where buffer is being filled by
 * something other than readLine()
 */
void readNameDef2(void)
{
 lin = buf;
 lin_copy = buf_copy;
 strcpy(lin_copy, lin);
 readNameDef();
}

/* generate a Name Definition
*/
void formatNameDef(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 vsnprintf(buf, LBUF_SIZE * sizeof(buf[0]), fmt, ap);
 readNameDef2();
}

/* generate a time line
*/
void formatTimeLine(int tim, char *fmt, ...)
{
 va_list ap;
 char *p = buf + sprintf(buf, "%02d:%02d:%02d ", tim / 3600, tim / 60 % 60, tim % 60);

 va_start(ap, fmt);
 vsnprintf(p, buf + (LBUF_SIZE * sizeof(buf[0])) - p, fmt, ap);
 readTimeLine2();
}

// Read Sequences head routines
// ---- --------- ---- --------

/* Generate a list of Period structures, based on the tone-specs
 * passed in (ac,av)
 */
void readSeqImm(int ac, char **av)
{
 char *p = buf;

 in_lin = 0;
 p += sprintf(p, "immediate:");
 while(ac-- > 0)
  p += sprintf(p, " %s", *av++);

 readNameDef2();
   
 strcpy(buf, "00:00 immediate");
 readTimeLine2();

 correctPeriods();
}

/* Read a list of sequence files, and generate a list of Period
 * structures
 */
void readSeq(int ac, char **av)
{
 // Setup a 'now' value to use for NOW in the sequence file
 now = calcNow();

 while(ac-- > 0)
 {
  char *fnam = *av++;
  int start = 1;

#ifdef SBG_GUI
  if(0 == strcmp("-", fnam))
  {
   scr_error("GUI not support stdin as input!");
  }
  in = fopen(fnam, "r");
#else
  in = (0 == strcmp("-", fnam))? stdin : fopen(fnam, "r");
#endif
  if(!in)
   scr_error("Can't open sequence file: %s", fnam);
      
  in_lin = 0;

  while(readLine())
  {
   char *p = lin;

   // Blank lines
   if(!*p)
    continue;

   // Look for options
   if(*p == '-')
   {
    if(!start) 
     scr_error("Options are only permitted at start of sequence file:\n  %s", p);
    handleOptions(p);
    continue;
   }

   // Check to see if it fits the form of <name>:<white-space>
   start = 0;
   if(!isalpha(*p))
    p = 0;
   else
   {
    while(isalnum(*p) || *p == '_' || *p == '-')
     p++;
    if(*p++ != ':' || !isspace(*p))
     p= 0;
   }

   if(p)
    readNameDef();
   else
    readTimeLine();
  }			// while(readLine())
      
  if(in != stdin)
  {
   fclose(in);
   in = NULL;
  }
 }
   
 correctPeriods();
}

// The Parser Mains
// --- ------ -----

/* Fill in all the correct information for the Periods, assuming
 * they have just been loaded using readTimeLine()
 */
void correctPeriods(void)
{
 // check for empty script
 if(NULL == per)
  scr_error("Empy script -- nothing to do");

 // Get times all correct
 {
  Period *pp = per;
  do
  {
   if(pp -> fi == -2)
   {
    pp -> tim = pp -> nxt -> tim;
    pp -> fi= -1;
   }

   pp = pp -> nxt;
  }
  while(pp != per);
 }

 // Make sure that the transitional periods each have enough time
 {
  Period *pp = per;

  do
  {
   if(pp -> fi == -1)
   {
    int per = t_per0(pp -> tim, pp -> nxt -> tim);

    if(per < fade_int)
    {
     int adj = (fade_int - per) / 2, adj0, adj1;

     adj0 = t_per0(pp -> prv -> tim, pp -> tim);
     adj0 = (adj < adj0)? adj : adj0;
     adj1 = t_per0(pp -> nxt -> tim, pp -> nxt-> nxt -> tim);
     adj1 = (adj < adj1)? adj : adj1;
     pp -> tim = (pp -> tim - adj0 + H24) % H24;
     pp-> nxt-> tim = (pp -> nxt -> tim + adj1) % H24;
    }
   }

   pp = pp -> nxt;
  }
  while(pp != per);
 }

 // Fill in all the voice arrays, and sort out details of
 // transitional periods
 {
  Period *pp = per;

  do
  {
   if(pp -> fi < 0)
   {
    int fo, fi;
    int a;
    int midpt= 0;

    Period *qq = (Period*)Alloc(sizeof(*qq));

    qq -> prv = pp;
    qq -> nxt = pp -> nxt;
    qq -> prv -> nxt = qq -> nxt -> prv = qq;

    qq -> tim = t_mid(pp -> tim, qq -> nxt -> tim);

    memcpy(pp -> v0, pp -> prv -> v1, sizeof(pp -> v0));
    memcpy(qq -> v1, qq -> nxt -> v0, sizeof(qq -> v1));

    // Special handling for bells
    for(a = 0; a < N_CH; a++)
    {
     if(pp -> v0[a].typ == 3 && pp -> fi != -3)
      pp -> v0[a].typ = 0;

     if(qq -> v1[a].typ == 3 && pp -> fi == -3)
      qq -> v1[a].typ = 0;
    }

    fo = pp -> prv -> fo;
    fi = qq -> nxt -> fi;

    // Special handling for -> slides:
    // always slide, and stretch slide if possible
    if(pp -> fi == -3)
    {
     fo = fi= 2;		// Force slides for ->
     for(a = 0; a < N_CH; a++)
     {
      Voice *vp = &(pp -> v0[a]);
      Voice *vq = &(qq -> v1[a]);

      if(vp -> typ == 0 && vq -> typ != 0 && vq -> typ != 3)
      {
       memcpy(vp, vq, sizeof(*vp)); vp->amp= 0;
      }
      else
       if(vp -> typ != 0 && vq -> typ == 0)
       {
        memcpy(vq, vp, sizeof(*vq));
        vq -> amp = 0;
       }
     }			// for(..a..)
    }

    memcpy(pp -> v1, pp -> v0, sizeof(pp -> v1));
    memcpy(qq -> v0, qq -> v1, sizeof(qq -> v0));

    for(a = 0; a < N_CH; a++)
    {
     Voice *vp = &(pp -> v1[a]);
     Voice *vq = &(qq -> v0[a]);

     if((fo == 0 || fi == 0) ||		// Fade in/out to silence
	(vp->typ != vq->typ) ||		// Different types
	((fo == 1 || fi == 1) &&	// Fade thru, but different pitches
	(vp->typ == 1 || vp->typ < 0) &&
	(vp->carr != vq->carr || vp->res != vq->res)))
     {
      vp -> amp = vq -> amp = 0;	// To silence
      midpt = 1;			// Definitely need the mid-point

      if(vq -> typ == 3)		// Special handling for bells
      {
       vq -> amp = qq -> v1[a].amp;
       qq -> nxt -> v0[a].typ = qq -> nxt -> v1[a].typ = 0;
      }
     }
     else		// Else smooth transition - for bells not so smooth
      if (vp->typ == 3)
      {
       qq -> v0[a].typ = qq -> v1[a].typ = 0;
      }
      else		// Else smooth transition
      {
       vp -> amp = vq -> amp = (vp -> amp + vq -> amp) / 2;
       if(vp -> typ == 1 || vp -> typ == 4 || vp -> typ < 0)
       {
        vp -> carr = vq -> carr = (vp -> carr + vq -> carr) / 2;
        vp -> res = vq -> res = (vp -> res + vq -> res) / 2;
       }
      }
    }			// for(..a..)

    // If we don't really need the mid-point, then get rid of it
    if(!midpt)
    {
     memcpy(pp -> v1, qq -> v1, sizeof(pp -> v1));
     qq -> prv -> nxt = qq -> nxt;
     qq -> nxt -> prv = qq -> prv;
     MFree(qq);
    }
    else
     pp = qq;
   }			// if(pp -> fi < 0)

   pp= pp->nxt;
  }
  while(pp != per);
 }

 // Clear out zero length sections, and duplicate sections
 {
  Period *pp;

  while(per != per->nxt)
  {
   pp = per;
   do
   {
    if(voicesEq(pp -> v0, pp -> v1) &&
	voicesEq(pp -> v0, pp -> nxt -> v0) &&
	voicesEq(pp -> v0, pp -> nxt -> v1))
     pp -> nxt -> tim = pp -> tim;

    if(pp -> tim == pp -> nxt -> tim)
    {
     if(per == pp)
      per = per -> prv;
     pp -> prv -> nxt = pp -> nxt;
     pp -> nxt -> prv = pp -> prv;
     MFree(pp);
     pp = 0;
     break;
    }
    pp = pp -> nxt;
   }
   while(pp != per);

   if(pp)
    break;
  }
 }

 // Make sure that the total is 24 hours only (not more !)
 if(per -> nxt != per)
 {
  int tot = 0;
  Period *pp = per;

  do
  {
   tot += t_per0(pp -> tim, pp -> nxt -> tim);
   pp = pp -> nxt;
  }
  while(pp != per);

  if(tot > H24)
  {
   warn("Total time is greater than 24 hours.  Probably two times are\n"
	"out of order.  Suspicious intervals are:\n");
   pp = per;
   do
   {
    if(t_per0(pp -> tim, pp -> nxt -> tim) >= H12)
     warn("  %02d:%02d:%02d -> %02d:%02d:%02d",
	pp -> tim % 86400000 / 3600000,
	pp -> tim % 3600000 / 60000,
	pp -> tim % 60000 / 1000,
	pp -> nxt -> tim % 86400000 / 3600000,
	pp -> nxt -> tim % 3600000 / 60000,
	pp -> nxt -> tim % 60000 / 1000);
    pp = pp -> nxt;
   }
   while(pp != per);
   scr_error("\nCheck the sequence around these times and try again");
  }
 }

 // Print the whole lot out
 if(opt_D)
 {
  Period *pp;

  if(per -> nxt != per)
   while(per -> prv -> tim < per -> tim)
    per = per -> nxt;

  pp = per;
  do
  {
   dispCurrPer(stdout);
   per = per -> nxt;
  }
  while(per != pp);
  scr_putc('\n', stdout);

  exitSBaGen(0);	// All done
 }
}

/* compare two Voices
*/
int voicesEq(Voice *v0, Voice *v1)
{
 int a = N_CH;

 while(a-- > 0)
 {
  if(v0 -> typ != v1 -> typ)
   return 0;
  switch(v0 -> typ)
  {
   case 1:
   case 4:
   default:
    if(v0 -> amp != v1 -> amp ||
	v0->carr != v1->carr ||
	v0->res != v1->res)
     return 0;
    break;

   case 2:
   case 5:
    if(v0 -> amp != v1 -> amp)
     return 0;
    break;

   case 3:
    if(v0 -> amp != v1 -> amp ||
	v0 -> carr != v1 -> carr)
     return 0;
    break;
  }
  v0++;
  v1++;
 }
 return 1;
}

/* Read a name definition
*/
void readNameDef(void)
{
 char *p, *q;
 NameDef *nd;
 int ch;

 if(!(p = getWord()))
  badSeq();

 q = strchr(p, 0) - 1;
 if(*q != ':')
  badSeq();

 *q = 0;
 for(q = p; *q; q++)
  if(!isalnum(*q) && *q != '-' && *q != '_')
   scr_error("Bad name \"%s\" in definition, line %d:\n  %s", p, in_lin, lin_copy);

 // Waveform definition ?
 if(0 == memcmp(p, "wave", 4) &&
	isdigit(p[4]) &&
	isdigit(p[5]) &&
	!p[6])
 {
  int ii = (p[4] - '0') * 10 + (p[5] - '0');
  int siz = ST_SIZ * sizeof(int);
  int *arr = (int*)Alloc(siz);
  double *dp0 = (double*)arr;
  double *dp1 = (double*)(siz + (char*)arr);
  double *dp = dp0;
  double dmax = 0, dmin = 1;
  int np;

  if(waves[ii])
   scr_error("Waveform %02d already defined, line %d:\n  %s",
	ii, in_lin, lin_copy);
  waves[ii] = arr;

  while((p = getWord()))
  {
   double dd;
   char dmy;

   if(1 != sscanf(p, "%lf %c", &dd, &dmy))
    scr_error("Expecting floating-point numbers on this waveform "
	"definition line, line %d:\n  %s",
	in_lin, lin_copy);
   if(dp >= dp1)
    scr_error("Too many samples on line (maximum %d), line %d:\n  %s",
	dp1 - dp0, in_lin, lin_copy);
   *dp++ = dd;
   if(dmax < dmin)
    dmin = dmax = dd;
   else
   {
    if(dd > dmax)
     dmax = dd;
    if(dd < dmin)
     dmin = dd;
   }
  }		// while((p = getWord()))
  dp1 = dp;
  np = dp1 - dp0;
  if(np < 2)
   scr_error("Expecting at least two samples in the waveform, line %d:\n  %s",
	in_lin, lin_copy);

  // Adjust to range 0-1
  for (dp = dp0; dp < dp1; dp++)
   *dp = (*dp - dmin) / (dmax - dmin);

  sinc_interpolate(dp0, np, arr);

  if(DEBUG_DUMP_WAVES)
  {
   int a;

   scr_printf("Dumping wave%02d:\n", ii);
   for(a = 0; a < ST_SIZ; a++)
    scr_printf("%d %g\n", a, arr[a] * 1.0 / ST_AMP);
  }
  return;
 }			// if(.."wave"..)

 // Must be block or tone-set, then, so put into a NameDef
 nd = (NameDef *)Alloc(sizeof(NameDef));
 nd -> name = StrDup(p);

 // Block definition ?
 if (*lin == '{')
 {
  BlockDef *bd, **prvp;

  if(!(p = getWord()) ||
	0 != strcmp(p, "{") ||
	0 != (p= getWord()))
   badSeq();

  prvp = &(nd -> blk);
    
  while(readLine())
  {
   if(*lin == '}')
   {
    if(!(p= getWord()) ||
	0 != strcmp(p, "}") ||
	0 != (p = getWord()))
     badSeq();
    if(!nd->blk)
     scr_error("Empty blocks not permitted, line %d:\n  %s", in_lin, lin_copy);
    nd -> nxt = nlist; nlist = nd;
    return;
   }

   if(*lin != '+')
    scr_error("All lines in the block must have relative time, line %d:\n  %s",
	in_lin, lin_copy);

   bd = (BlockDef *) Alloc(sizeof(*bd));
   *prvp = bd; prvp = &(bd -> nxt);
   bd -> lin = StrDup(lin);
  }

  // Hit EOF before }
  scr_error("End-of-file within block definition (missing '}')");
 }		// if('{')

 // Normal line-definition
 for(ch = 0; ch < N_CH && (p = getWord()); ch++)
 {
  char dmy;
  double amp, carr, res;
  int wave;

  // Interpret word into Voice nd->vv[ch]
  if(0 == strcmp(p, "-"))
   continue;
  if(1 == sscanf(p, "pink/%lf %c", &amp, &dmy))
  {
   nd -> vv[ch].typ = 2;
   nd -> vv[ch].amp = AMP_DA(amp);
   continue;
  }
  if(2 == sscanf(p, "bell%lf/%lf %c", &carr, &amp, &dmy))
  {
   nd -> vv[ch].typ = 3;
   nd -> vv[ch].carr = carr;
   nd -> vv[ch].amp = AMP_DA(amp);
   continue;
  }
  if(1 == sscanf(p, "mix/%lf %c", &amp, &dmy))
  {
   nd -> vv[ch].typ = 5;
   nd -> vv[ch].amp = AMP_DA(amp);
   mix_flag = 1;
   continue;
  }
  if(4 == sscanf(p, "wave%d:%lf%lf/%lf %c", &wave, &carr, &res, &amp, &dmy))
  {
   if(wave < 0 || wave >= 100)
    scr_error("Only wave00 to wave99 is permitted at line: %d\n  %s", in_lin, lin_copy);
   if(!waves[wave])
    scr_error("Waveform %02d has not been defined, line: %d\n  %s", wave, in_lin, lin_copy);
   nd -> vv[ch].typ  = -1 - wave;
   nd -> vv[ch].carr = carr;
   nd -> vv[ch].res  = res;
   nd -> vv[ch].amp  = AMP_DA(amp);
   continue;
  }
  if(3 == sscanf(p, "%lf%lf/%lf %c", &carr, &res, &amp, &dmy))
  {
   nd -> vv[ch].typ  = 1;
   nd -> vv[ch].carr = carr;
   nd -> vv[ch].res  = res;
   nd -> vv[ch].amp  = AMP_DA(amp);
   continue;
  }
  if(2 == sscanf(p, "%lf/%lf %c", &carr, &amp, &dmy))
  {
   nd -> vv[ch].typ  = 1;
   nd -> vv[ch].carr = carr;
   nd -> vv[ch].res  = 0;
   nd -> vv[ch].amp  = AMP_DA(amp);
   continue;
  }
  if(3 == sscanf(p, "spin:%lf%lf/%lf %c", &carr, &res, &amp, &dmy))
  {
   nd -> vv[ch].typ  = 4;
   nd -> vv[ch].carr = carr;
   nd -> vv[ch].res  = res;
   nd -> vv[ch].amp  = AMP_DA(amp);
   continue;
  }
  badSeq();
 }		// for(..ch..)
 nd -> nxt = nlist;
 nlist = nd;
}  

/* Read a time-line of either type
*/
void readTimeLine(void)
{
 char *p, *tim_p;
 int nn;
 int fo, fi;
 Period *pp;
 NameDef *nd;
 int tim, rtim = 0;		// rtim = 0 -- to eliminate GCC warning
 static int last_abs_time = -1;

 if(!(p = getWord()))
  badSeq();

 tim_p = p;
  
 // Read the time represented
 tim = -1;
 if(0 == memcmp(p, "NOW", 3))
 {
  last_abs_time = tim = now;
  p += 3;
 }

 while(*p)
 {
  if(*p == '+')
  {
   if(tim < 0)
   {
    if(last_abs_time < 0)
     scr_error("Relative time without previous absolute time, line %d:\n  %s",
	in_lin, lin_copy);
    tim = last_abs_time;
   }
   p++;
  }
  else
   if(tim != -1)
    badTime(tim_p);

  if(0 == (nn = readTime(p, &rtim)))
   badTime(tim_p);
  p += nn;

  if(tim == -1) 
   last_abs_time = tim = rtim;
  else
   tim = (tim + rtim) % H24;
 }		// while(*p)

 if(fast_tim0 < 0)			// First time
  fast_tim0 = tim;
 fast_tim1= tim;			// Last time
      
 if(!(p = getWord()))
  badSeq();

 fi = fo = 1;
 if(!isalpha(*p))
 {
  switch(p[0])
  {
   case '<':
    fi = 0;
    break;

   case '-':
    fi = 1;
    break;

   case '=':
    fi = 2;
    break;

   default:
    badSeq();
  }

  switch(p[1])
  {
   case '>':
    fo = 0;
    break;

   case '-':
    fo = 1;
    break;

   case '=':
    fo = 2;
    break;

   default:
    badSeq();
  }

  if(p[2])
   badSeq();

  if(!(p = getWord()))
   badSeq();
 }			// if(!isalpha(*p))

 for(nd = nlist; nd && 0 != strcmp(p, nd -> name); nd = nd -> nxt)
  /* NOTHING */ ;
 if(!nd)
  scr_error("Name \"%s\" not defined, line %d:\n  %s", p, in_lin, lin_copy);

 // Check for block name-def
 if(nd -> blk)
 {
  char *prep = StrDup(tim_p);	// Put this at the start of each line
  BlockDef *bd = nd -> blk;

  while(bd)
  {
   lin = buf;
   lin_copy = buf_copy;
   sprintf(lin, "%s%s", prep, bd -> lin);
   strcpy(lin_copy, lin);
   readTimeLine();		// This may recurse, and that's why we're StrDuping the string
   bd = bd -> nxt;
  }
  MFree(prep);
  return;
 }
      
 // Normal name-def
 pp = (Period *)Alloc(sizeof(*pp));
 pp -> tim = tim;
 pp -> fi  = fi;
 pp -> fo  = fo;

 memcpy(pp -> v0, nd -> vv, N_CH * sizeof(Voice));
 memcpy(pp -> v1, nd -> vv, N_CH * sizeof(Voice));

 if(!per)
  per = pp -> nxt = pp -> prv = pp;
 else
 {
  pp -> nxt = per;
  pp -> prv = per -> prv;
  pp -> prv -> nxt = pp -> nxt -> prv = pp;
 }

 // Automatically add a transitional period
 pp = (Period *)Alloc(sizeof(*pp));
 pp -> fi  = -2;		// Unspecified transition
 pp -> nxt = per;
 pp -> prv = per -> prv;
 pp -> prv -> nxt = pp -> nxt -> prv = pp;

 if(0 != (p = getWord()))
 {
  if(0 != strcmp(p, "->"))
   badSeq();
  pp -> fi = -3;		// Special '->' transition
  pp -> tim = tim;
 }
}

/* read time stamp
*/
int readTime(char *p, int *timp)	// Rets chars consumed, or 0 error
{
 int nn, hh, mm, ss;

 if(3 > sscanf(p, "%2d:%2d:%2d%n", &hh, &mm, &ss, &nn))
 {
  ss = 0;
  if(2 > sscanf(p, "%2d:%2d%n", &hh, &mm, &nn))
   return 0;
 }

 if(hh < 0 || hh >= 24 ||
	mm < 0 || mm >= 60 ||
	ss < 0 || ss >= 60)
  return 0;

 *timp = ((hh * 60 + mm) * 60 + ss) * 1000;
 return nn;
}

// Handling pre-programmed sequences
// -------- -------------- ---------

/* general pre-programmed routine
*/
void readPreProg(int ac, char **av)
{
 if(ac < 1)
  scr_error("Expecting a pre-programmed sequence description.  Examples:"
	NL "  drop 25ds+ pink/30"
	NL "  drop 25gs+/2 mix/60");

 // Handle 'drop'
 if(0 == strcmp(av[0], "drop"))
 {
  ac--;
  av++;
  create_drop(ac, av);
  return;
 }

 // Handle 'slide'
 if(0 == strcmp(av[0], "slide"))
 {
  ac--;
  av++;
  create_slide(ac, av);
  return;
 }

 scr_error("Unknown pre-programmed sequence type: %s", av[0]);
}

/* Generate a p-drop sequence
 * Credits: Jonathan Bisson created the first version of this C
 * code.  This is a rewrite to make it fit with the rest of the
 * code better.
 */
void create_drop(int ac, char **av)
{
 char *fmt;
 char *p, *q;
 int a;
 int slide, n_step, islong, wakeup;
 double carr, amp, c0, c2;
 double beat_target;
 double *beat;
 int len, len0 = 1800, len1 = 1800, len2 = 180;
 int steplen, end;
 char extra[256];
 static double beat_targets[] =
 {
  4.4, 3.7, 3.1, 2.5, 2.0, 1.5, 1.2, 0.9, 0.7, 0.5, 0.4, 0.3
 };
   
 // Pick up optional time-spec
 if(ac < 1)
  bad_drop();

 if(av[0][0] == 't')
 {
  double v0, v1, v2;
  char dmy;

  if(3 != sscanf(av[0] + 1, "%lf,%lf,%lf %c", &v0, &v1, &v2, &dmy))
   bad_drop();
  len0 = 60 * (int)v0;		// Whole minutes only
  len1 = 60 * (int)v1;
  len2 = 60 * (int)v2;
  ac--;
  av++;
 }

 // Handle argument list
 if(ac < 1)
  bad_drop();
 fmt = *av++;
 ac--;
 p = extra;
 *p = 0;
 while(ac > 0)
 {
  if(p + strlen(av[0]) + 2 > extra + sizeof(extra))
   scr_error("Too many extra tone-specs after -p drop");
  p += sprintf(p, " %s", av[0]);
  ac--;
  av++;
 }

 // Scan the format
 carr = 200 - 2 * strtod(fmt, &p);
 if(p == fmt || carr < 0)
  bad_drop();

 a = tolower(*p) - 'a';
 p++;
 if(a < 0 || a >= sizeof(beat_targets) / sizeof(beat_targets[0]))
  bad_drop();
 beat_target = beat_targets[a];

 slide = 0;
 steplen = 180;
 if(*p == 's')
 {
  p++;
  slide= 1;
  steplen= 60;
 }
 else
  if(*p == 'k')
  {
   p++;
   steplen= 60;
  }
 n_step = 1 + (len0 - 1) / steplen;	// Round up
 beat = Alloc(n_step * sizeof(double));
 len0 = n_step * steplen;
 if(!slide)
  len1 = (1 + (len1 - 1) / steplen) * steplen;

 islong = 0;
 if(*p == '+')
 {
  islong = 1;
  p++;
 }

 wakeup = 0;
 if(*p == '^' || *p == '!')		// '^' -- metasymbol in Windows
 {
  wakeup = 1;
  p++;
 }

 amp = 1.0;
 if(*p == '/')
 {
  p++;
  q = p;
  amp = strtod(p, &p);
  if(p == q)
   bad_drop();
 }

 while(isspace(*p))
  p++;
 if(*p)
  scr_error("Trailing rubbish after -p drop spec: \"%s\"", p);

 // Sort out carriers
 len = islong? len0 + len1 : len0;
 c0 = carr + 5.0;
 c2 = carr;

 // Calculate beats
 for(a = 0; a < n_step; a++)
  beat[a]= 10 * exp(log(beat_target / 10.0) * a / (n_step - 1));

 // Display summary
 warn("DROP summary:");
 if(slide)
 {
  warn(" Carrier slides from %gHz to %gHz over %d minutes",
	c0, c2, len / 60);
  warn(" Beat frequency slides from %gHz to %gHz over %d minutes",
	beat[0], beat[n_step - 1], len0 / 60);
 }
 else
 {
  warn(" Carrier steps from %gHz to %gHz over %d minutes",
	c0, c2, len / 60);
  warn(" Beat frequency steps from %gHz to %gHz over %d minutes:",
	beat[0], beat[n_step - 1], len0 / 60);
  scr_fprintf(stderr, "   ");
  for(a = 0; a < n_step; a++)
   scr_fprintf(stderr, " %.2f", beat[a]);
  scr_fprintf(stderr, "\n");
 }
 if(wakeup)
 {
  warn(" Final wake-up of %d minutes, to return to initial frequencies", len2/60);
 }

 // Start generating sequence
 handleOptions("-SE");
 in_lin = 0;

 formatNameDef("off: -");
 formatTimeLine(86395, "== off ->");		// 23:59:55
   
 if(slide)
 {
  // Slide version
  for(a = 0; a < n_step; a++)
  {
   int tim = a * len0 / (n_step - 1);

   formatNameDef("ts%02d: %g+%g/%g %s",
	a, c0 + (c2-c0) * tim * 1.0 / len, beat[a], amp, extra);
   formatTimeLine(tim, "== ts%02d ->", a);
  }

  if(islong)
  {
   formatNameDef("tsend: %g+%g/%g %s",
	c2, beat[n_step - 1], amp, extra);
   formatTimeLine(len, "== tsend ->");
  }
  end = len;
 }
 else
 {
  // Step version
  int lim = len / steplen;
  int stepslide = steplen < 90 ? 5 : 10;	// Seconds slide between steps

  for(a = 0; a < lim; a++)
  {
   int tim0 = a * steplen;
   int tim1 = (a + 1) * steplen;

   formatNameDef("ts%02d: %g+%g/%g %s",
	a, c0 + (c2-c0) * tim1/len, beat[(a >= n_step)? n_step - 1 : a],
	amp, extra);
   formatTimeLine(tim0, "== ts%02d ->", a);
   formatTimeLine(tim1-stepslide, "== ts%02d ->", a);
  }
  end = len - stepslide;
 }
   
 // Wake-up and ending
 if(wakeup)
 {
  formatNameDef("tswake: %g+%g/%g %s", c0, beat[0], amp, extra);
  formatTimeLine(end+len2, "== tswake ->");
  end += len2;
 }
 formatTimeLine(end+10, "== off");

 MFree(beat);

 correctPeriods();
}

/* Generate a -p slide sequence
 * The idea of this is to hold the beat frequency constant, but
 * to slide down through the carrier frequencies from about 200Hz.
 *
 * -p slide [t<duration-minutes>] <carr>+<beat>/<amp> [extra tone-sets]
 */
void create_slide(int ac, char **av)
{
 int len = 1800;
 char *p, dmy;
 double val, c0, c1, beat, amp;
 char extra[256];

 // Handle arguments
 if(ac < 1)
  bad_slide();

 if(av[0][0] == 't')
 {
  val = strtod(av[0] + 1, &p);
  if(p == av[0] + 1 || *p)
   bad_slide();
  len = (int)(60.0 * val);
  ac--;
  av++;
 }

 if(ac < 1)
  bad_slide();
 if(3 != sscanf(av[0], "%lf%lf/%lf %c", &c0, &beat, &amp, &dmy))
  bad_slide();

 c1 = beat / 2.0;
 ac--;
 av++;

 // Gather 'extra'
 p = extra;
 *p= 0;
 while(ac > 0)
 {
  if(p + strlen(av[0]) + 2 > extra + sizeof(extra))
   scr_error("Too many extra tone-specs after -p slide");
  p += sprintf(p, " %s", av[0]);
  ac--;
  av++;
 }

 // Summary
 warn("SLIDE summary:");
 warn(" Sliding carrier from %gHz to %gHz over %g minutes",
	c0, c1, len / 60.0);
 warn(" Holding beat constant at %gHz", beat);

 // Generate sequence
 handleOptions("-SE");
 in_lin = 0;

 formatNameDef("off: -");
 formatTimeLine(86395, "== off ->");		// 23:59:55
 formatNameDef("ts0: %g%+g/%g %s", c0, beat, amp, extra);
 formatTimeLine(0, "== ts0 ->");
 formatNameDef("ts1: %g%+g/%g %s", c1, beat, amp, extra);
 formatTimeLine(len, "== ts1 ->");
 formatTimeLine(len+10, "== off");

 correctPeriods();
}

/* The END...
*/

