/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is THE MAIN SBaGen file

#include "sbagenr.h"


/* The	M A I N  function
*/
#ifdef	SBG_GUI
int gui_main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
{
 short test= 0x1100;
 int rv;
 char *p;

 ClearStaticArrays();
 InitAlloc();

 pdir = StrDup(argv[0]);
 p = strchr(pdir, 0);
 while (p > pdir && p[-1] != '/' && p[-1] != '\\') *--p= 0;

 argc--; argv++;
 init_sin_table();
 bigendian = ((char*)&test)[0] != 0;

 // Process all the options
 rv = scanOptions(&argc, &argv);

 if(argc < 1)
  usage();

 if (rv == 'i')
 {
  // Immediate mode
  readSeqImm(argc, argv);
 }
 else
 {
  if (rv == 'p')
  {
   // Pre-programmed sequence
   readPreProg(argc, argv);
  }
  else
  {
   // Sequenced mode -- sequence may include options, so options
   // are not settled until below this point
   if (argc < 1)
    usage();
   readSeq(argc, argv);
  }
 }
   
 if(opt_W && !opt_o && !opt_O)
  scr_error("Use -o or -O with the -W option");
 if(opt_W && opt_L < 0 && !opt_E)
  scr_error("Use -L or -E with -W option to give the length of the WAV file");
   
 mix_in = 0;
 if(opt_M || opt_m)
 {
  char *p;
  char tmp[4];

  ib_stream_type = IBST_RAW;

  if(opt_M)
  {
   mix_in = stdin;
   tmp[0] = 0;
  } 
  if(opt_m)
  {
   int tmix_cnt = -1;

   // Pick up #<digits> on end of filename
   p = strchr(opt_m, 0);
   if(p > opt_m && isdigit(p[-1]))
   {
    tmix_cnt = 0;

    while (p > opt_m && isdigit(p[-1]))
     tmix_cnt= tmix_cnt * 10 + *--p - '0';

    if(p > opt_m && (p[-1] == '#' || p[-1] == '!'))
     *--p = 0;
    else
    {
     p = strchr(opt_m, 0);
     tmix_cnt = -1;
    }
   }
   // p points to end of filename (NULL)
   if(mix_cnt < 0)	// -l n override file.ogg#n
    mix_cnt = tmix_cnt;

   // Open file
   raw_init_pos = 0;
   wav_data_len = 0;
   wav_data_cnt = 0;

   mix_in = fopen(opt_m, "rb");
   if(!mix_in && opt_m[0] != '/')
   {
    int len = strlen(opt_m) + strlen(pdir) + 1;
    char *tmp = ALLOC_ARR(len, char);

    strcpy(tmp, pdir);
    strcat(tmp, opt_m);
    mix_in = fopen(tmp, "rb");
    MFree(tmp);
   }
   if(!mix_in)
    scr_error("Can't open -m option mix input file: %s", opt_m);

   // Pick up extension
   if(p - opt_m >= 4 && p[-4] == '.')
   {
    tmp[0] = tolower(p[-3]);
    tmp[1] = tolower(p[-2]);
    tmp[2] = tolower(p[-1]);
    tmp[3] = 0;
   }
  }
  if(0 == strcmp(tmp, "wav"))	// Skip header on WAV files
   find_wav_data_start(mix_in);
  if(0 == strcmp(tmp, "ogg"))
  {
#ifdef OGG_DECODE
   ib_stream_type = IBST_OGG;
   if(opt_mrep)
    warn("-m+ repeat tag ignored for OGG-files");
   ogg_init();
#else
   scr_error("Sorry: Ogg support wasn't compiled into this executable");
#endif
  }
  if(0 == strcmp(tmp, "mp3"))
  {
#ifdef MP3_DECODE
   ib_stream_type = IBST_MP3;
   if(opt_mrep)
    warn("-m+ repeat tag ignored for MP3-files");
   mp3_init();
#else
   scr_error("Sorry: MP3 support wasn't compiled into this executable");
#endif
  }
  // If this is a raw/wav data stream, setup a 256*1024-int
  // buffer (3s@44.1kHz)
  if(IBST_RAW == ib_stream_type)
   inbuf_start(raw_mix_in, 256 * 1024);
 }				// if(opt_M || opt_m)

 loop();

 exitSBaGen(0);			// free all resources
 return 0;			// decorate
}

/* The Main Loop
*/
void loop(void)
{	
 int c, cnt;
 int err;			// Error to add to 'now' until next cnt==0
 int fast = fast_mult != 0;
 int vfast = fast_mult > 20;	// Very fast - update status line often
 int utime = 0;
 int now_lo = 0;		// Low-order 16 bits of 'now' (fractional)
 int err_lo = 0;
 int ms_inc;

 setup_device();

 spin_carr_max = 127.0 / 1E-6 / out_rate;
 cnt = 1 + 1999 / out_buf_ms;	// Update every 2 seconds or so
 now= opt_S ? fast_tim0 : calcNow();
 if(opt_T != -1)
  now= opt_T;
 err = fast? out_buf_ms * (fast_mult - 1) : 0;
 if(opt_L)
  byte_count = out_bps * (S64)(opt_L * 0.001 * out_rate);
 if(opt_E)
  byte_count= out_bps * (S64)(t_per0(now, fast_tim1) * 0.001 * out_rate /
	(fast? fast_mult : 1));

 // Do byte-swapping if bigendian and outputting to a file or stream
 if((opt_O || opt_o) && out_mode == 1 && bigendian)
  out_mode = 2;

 if(opt_W)
  writeWAV();

 if(!opt_Q)
  scr_putc('\n', stderr);
 corrVal(0);		// Get into correct period
 dispCurrPer(stderr);	// Display
 status(0);
  
 for(;;)					// ++LOOP
 {
  for(c = 0; c < cnt; c++)
  {
   CHECK_EXIT_CONDITIONS;
   corrVal(1);
   outChunk();
   ms_inc = out_buf_ms + err;
   now_lo += out_buf_lo + err_lo;
   if(now_lo >= 0x10000)
   {
    ms_inc += now_lo >> 16;
    now_lo &= 0xFFFF;
   }
   now += ms_inc;
   if(now > H24)
    now -= H24;
   if(vfast && (c & 1))
    status(0);
  }

  if(fast)
  {
   if(!vfast)
    status(0);
  }
  else
  {
   // Synchronize with real clock, gently over the next second or so
   char buf[32];
   int diff = calcNow() - now;

   if(abs(diff) > H12)
    diff = 0;
   sprintf(buf, "(%d)", diff);

   err_lo = diff * 0x10000 / cnt;
   err = err_lo >> 16;
   err_lo &= 0xFFFF;

   if(DEBUG_CHK_UTIME)
   {
    int prev = utime;

    utime = userTime();
    sprintf(buf, "%d ticks", utime-prev);	// Replaces standard message
   }
   status(buf);
  }
 }						// --LOOP
}

/* exit with full cleanup (may be wrong for U*X)
*/
void exitSBaGen(int code)
{
 // thread cleanup
 inbuf_stop();

 // cleanup audio output (include out file)
 audio_cleanup();

 // close script file
 if(in && in != stdin)
 {
  fclose(in);
  in = NULL;
 }

 // close mix file
 if(mix_in && mix_in != stdin)
 {
  fclose(mix_in);
  mix_in = NULL;
 }

 // free all memory
 freeAll();

 // all allocated resources seems set free
#ifndef SBG_GUI
 exit(code);
#else
 scr_exit_notify(code);
 _endthreadex((unsigned)code);
#endif
}

/* The END...
*/

