/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

#ifndef _INC_SBAGENR
#define _INC_SBAGENR

#ifdef __cplusplus
extern "C" {
#endif

#define VERSION "1.4.4R"
// The history
// --- -------
// 1.4.3R -- 27.04.07 -- split sources and reformat code; thread start/
//	/stop for windows fixed
// 1.4.4R -- free all resources at exit (exit(int) replaced; -p drop timing fixed,
//	-p drop '!' as synonim for '^' added. Synch with read thread for windows,
//	add -m+ for endless repeat wav/raw mix,
//	add support for compilation as win32 GUI application
//	(ASCII/ANSI and UNICODE) (all the stdio screen output functions
//	replaced),
//	add support for compilation as Win CE (PPC 2003 and
//	Windows Mobile 2005) w/o any additional libs),
//	+Windows Vista Support,
//	fix -p drop with non-standard timing
//	fix working with *.wav files (but thw wav's different from
//	16 bit/stereo/PCM stay handlend incorectly)
//	some minor changes.
//	*Note, that MP3/OGG mix not supported yet!*
//	Now there are 4 projects for Windows:
//	sbagenr.exe -- plain command line utility;
//	sbagenw.exe -- WIN32 GUI ASCII/ANSI appl;
//	sbagenwu.exe -- same, but using only UNICODE system calls;
//	sbagene.exe -- PPC versions for many different CPU
//	NOTE: sbagenw & sbagenwu was not planning as separate programs,
//	its wrote as approaches for the PPC sbagene.
//	NOTE: all the project copiled from the same code tree.
//	add -B bufSiz,bufCount
// -history

// This should be built with one of the following target macros
// defined, which selects options for that platform, or else with some
// of the individual named flags #defined as listed later.
//
//  T_LINUX	To build the LINUX version with /dev/dsp support
//  T_MINGW	To build for Windows using MinGW and Win32 calls
//  T_MSVC	To build for Windows using MSVC and Win32 calls
//  T_MACOSX	To build for MacOSX using CoreAudio
//  T_POSIX	To build for simple file output on any Posix-compliant OS
//
// Ogg and MP3 support is handled separately from the T_* macros.

// Define OSS_AUDIO to use /dev/dsp for audio output
// Define WIN_AUDIO to use Win32 calls
// Define MAC_AUDIO to use Mac CoreAudio calls
// Define NO_AUDIO if no audio output device is usable
// Define UNIX_TIME to use UNIX calls for getting time
// Define WIN_TIME to use Win32 calls for getting time
// Define ANSI_TTY to use ANSI sequences to clear/redraw lines
// Define UNIX_MISC to use UNIX calls for various miscellaneous things
// Define WIN_MISC to use Windows calls for various miscellaneous things
// Define EXIT_KEY to require the user to hit RETURN before exiting after error

// Define OGG_DECODE to include OGG support code
// Define MP3_DECODE to include MP3 support code

#ifdef T_LINUX
#define OSS_AUDIO
#define UNIX_TIME
#define UNIX_MISC
#define ANSI_TTY
#endif

#ifdef T_MINGW
#define WIN_AUDIO
#define WIN_TIME
#define WIN_MISC
#define EXIT_KEY
#endif

#ifdef T_MSVC
#define WIN_AUDIO
#define WIN_TIME
#define WIN_MISC
#define EXIT_KEY
// to eliminate lots of stupid MS Visual Studio 8.0+ warnings
#define _CRT_SECURE_NO_DEPRECATE
#endif

#ifdef T_MACOSX
#define MAC_AUDIO
#define UNIX_TIME
#define UNIX_MISC
#define ANSI_TTY
#endif

#ifdef T_POSIX
#define NO_AUDIO
#define UNIX_TIME
#define UNIX_MISC
#endif

// Make sure NO_AUDIO is set if necessary
#ifndef OSS_AUDIO
#ifndef MAC_AUDIO
#ifndef WIN_AUDIO

#define NO_AUDIO

#endif
#endif
#endif

// Make sure one of the _TIME macros is set
#ifndef UNIX_TIME
#ifndef WIN_TIME

#error UNIX_TIME or WIN_TIME not defined.  Maybe you did not define one of T_LINUX/T_MINGW/T_MACOSX/etc ?

#endif
#endif

// Make sure one of the _MISC macros is set
#ifndef UNIX_MISC
#ifndef WIN_MISC
#error UNIX_MISC or WIN_MISC not defined.  Maybe you did not define one of T_LINUX/T_MINGW/T_MACOSX/etc ?
#endif
#endif

#include <stdio.h>
#include <math.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#ifndef UNDER_CE
#include <errno.h>
#endif

#include <ctype.h>

#ifndef UNDER_CE
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#include <time.h>

#ifdef T_MSVC

#ifndef UNDER_CE
#include <io.h>
#endif

#define	write		_write
#define vsnprintf	_vsnprintf
#define INLINE		__inline
#define	THREAD		__stdcall
typedef	__int64	S64;

#else

#include <unistd.h>
#include <sys/time.h>
typedef long long S64;
#define INLINE	static inline	/* GCC C extention */
#define	THREAD

#endif

#ifdef T_MINGW
#define vsnprintf	_vsnprintf
#endif

#ifdef OSS_AUDIO
#include <linux/soundcard.h>	// WAS: #include <sys/soundcard.h>
#define	AU_NO_HANDLE	(9999)
#endif

#ifdef WIN_AUDIO
#include <windows.h>
#include <mmsystem.h>
#define	AU_NO_HANDLE	(INVALID_HANDLE_VALUE)
#endif

#ifdef MAC_AUDIO
#include <Carbon.h>
#include <CoreAudio/CoreAudio.h>
#define	AU_NO_HANDLE	(-9999)
#endif

#ifdef UNIX_TIME
#include <sys/ioctl.h>
#include <sys/times.h>
#endif

#ifdef UNIX_MISC
#include <pthread.h>
#endif

#ifdef WIN_MISC
#ifndef UNDER_CE
#include <process.h>		// _beginthredex / _endthreadex
#endif
#endif

// usage _vsnprintf vs vsnprintf for T_MINGW and T_MSVC moved to xxx_stdio.h
#ifdef SBG_GUI			// GUI Version stuff
#include "sbagenw/gui_stdio.h"
#else
#include "cmd_stdio.h"
#endif

/* THE TUNING -- See it cerryfully!
 * --- ------ -- --- -- -----------
 */
#define DEBUG_CHK_UTIME		(0)	// Check how much user time is being consumed
#define DEBUG_DUMP_WAVES	(0)	// Dump out wave tables (to plot with gnuplot)
#define DEBUG_DUMP_AMP		(0)	// Dump output amplitude to stdout per chunk
#define N_CH			(16)	// !!Number of channels!!
#define	MAX_ARGC		(32)	// !!max no of arguments in command line!!

#define	DEBUG_MEM		(0)	// dump memory allocator info
#define	DEBUG_CLEANUP		(0)	// output info about destroying objects

// new sizes
#define	AMP_ADJ_SIZ		(16)
#define	LBUF_SIZE		(4096)

// buffers sizes
#ifdef WIN_AUDIO

#define BUFFER_COUNT		(8)		/* default */
#define BUFFER_SIZE		(8192*4)	/* default */

#endif

#ifdef MAC_AUDIO

#define BUFFER_COUNT		(8)
#define BUFFER_SIZE		(4096*4)

#endif


/* general types (strict C-style)
*/
struct tagVoice
{
 int typ;			// Voice type: 0 off, 1 binaural, 2 pink noise, 3 bell, 4 spin,
				//   5 mix, 6 mixspin, 7 mixbeat, -1 to -100 wave00 to wave99
 double amp;			// Amplitude level (0-4096 for 0-100%)
 double carr;			// Carrier freq (for binaural/bell), width (for spin)
 double res;			// Resonance freq (-ve or +ve) (for binaural/spin)
};

struct tagChannel
{
 struct tagVoice v;		// Current voice setting (updated from current period)
 int typ;			// Current type: 0 off, 1 binaural, 2 pink noise, 3 bell, 4 spin,
   				//   5 mix, 6 mixspin, 7 mixbeat, -1 to -100 wave00 to wave99
 int amp, amp2;			// Current state, according to current type
 int inc1, off1;		//  ::  (for binaural tones, offset + increment into sine 
 int inc2, off2;		//  ::   table * 65536)
};

struct tagPeriod
{
 struct tagPeriod *nxt, *prv;	// Next/prev in chain
 int tim;			// Start time (end time is ->nxt->tim)
 struct tagVoice v0[N_CH], v1[N_CH]; // Start and end voices
 int fi, fo;			// Temporary: Fade-in, fade-out modes
};

struct tagNameDef
{
 struct tagNameDef *nxt;
 char *name;			// Name of definition
 struct tagBlockDef *blk;	// Non-zero for block definition
 struct tagVoice vv[N_CH];	// Voice-set for it (unless a block definition)
};

struct tagBlockDef
{
 struct tagBlockDef *nxt;	// Next in chain
 char *lin;			// StrDup'd line
};

struct tagAmpAdj
{
 double freq, adj;
};

struct tagNoise
{
 int val;			// Current output value
 int inc;			// Increment
};

// Memory list stuff (1.4.4R+)
struct tagHeapList		// list of memory, allocated with Alloc()
{
 struct tagHeapList *nxt;	// the LAST == NULL
 struct tagHeapList *prv;	// the FIRST == NULL
 void *memptr;			// the pointer to allocated block
};

/* the useful typedefs
*/
typedef struct tagChannel Channel;
typedef struct tagVoice Voice;
typedef struct tagPeriod Period;
typedef struct tagNameDef NameDef;
typedef struct tagBlockDef BlockDef;
typedef struct tagAmpAdj AmpAdj;
typedef struct tagNoise Noise;
typedef unsigned char uchar;
typedef unsigned int uint;	// there was #define here

typedef struct tagHeapList HeapList;	// 1.4.4R+

/*
 * The GLOBAL Variables (file 'sbvars.c')
 * --- ------ --------- ----- -----------
 */

extern int *sin_table;

extern int *waves[];			// Pointers are either 0 or point to a sin_table[]-style array of int

extern Channel chan[];			// Current channel states
extern int now;				// Current time (milliseconds from midnight)
extern Period *per;			// Current period
extern NameDef *nlist;			// Full list of name definitions

extern int *tmp_buf;			// Temporary buffer for 20-bit mix values
extern short *out_buf;			// Output buffer
extern int out_bsiz;			// Output buffer size (bytes)
extern int out_blen;			// Output buffer length (samples) (1.0* or 0.5* out_bsiz)
extern int out_bps;			// Output bytes per sample (2 or 4)
extern int out_buf_ms;			// Time to output a buffer-ful in ms
extern int out_buf_lo;			// Time to output a buffer-ful, fine-tuning in ms/0x10000
extern int out_fd;			// Output file descriptor
FILE *out_file;		;		// FILE for out_fd
extern int out_rate;			// Sample rate
extern int out_rate_def;		// Sample rate is default value, not set by user
extern int out_mode;			// Output mode: 0 unsigned char[2], 1 short[2], 2 swapped short[2]
extern int out_prate;			// Rate of parameter change (for file and pipe output only)
extern int fade_int;			// Fade interval (ms)
extern FILE *in;			// Input sequence file
extern int in_lin;			// Current input line
extern char buf[];			// Buffer for current line
extern char buf_copy[];			// Used to keep unmodified copy of line
extern char *lin;			// Input line (uses buf[])
extern char *lin_copy;			// Copy of input line
extern double spin_carr_max;		// Maximum 'carrier' value for spin (really max width in us)

extern int fast_tim0;			// First time mentioned in the sequence file (for -q and -S option)
extern int fast_tim1;			// Last time mentioned in the sequence file (for -E option)
extern int fast_mult;			// 0 to sync to clock (adjusting as necessary), or else sync to
					//  output rate, with the multiplier indicated
extern S64 byte_count;			// Number of bytes left to output, or -1 if unlimited
extern int tty_erase;			// Chars to erase from current line (for ESC[K emulation)

extern int opt_D;
extern int opt_M;
extern int opt_Q;
extern int opt_S;
extern int opt_E;
extern int opt_W;
extern int opt_O;
extern int opt_L;			// Length in ms, or -1
extern int opt_T;			// Start time in ms, or -1
extern char *opt_o;			// File name to output to, or 0
extern char *opt_m;			// File name to read mix data from, or 0
extern int opt_mrep;			// repeat wav/raw file
extern char *opt_d;			// Output device

extern FILE *mix_in;			// Input stream for mix sound data, or 0
extern int mix_cnt;			// Version number from mix filename (#<digits>), or -1
extern int bigendian;			// Is this platform Big-endian?
extern int mix_flag;			// Has 'mix/*' been used in the sequence?
extern long raw_init_pos;		// init position of raw/wav stream
extern int wav_data_len;		// lenght of wav data chunk
extern int wav_data_cnt;		// counter of wav data

extern int opt_c;			// Number of -c option points provided (max 16)
extern AmpAdj ampadj[];			// List of maximum 16 (freq,adj) pairs, freq-increasing order

extern char *pdir;			// Program directory (used as second place to look for -m files)

// Noise stuff
extern Noise ntbl[];			// variables
extern int nt_off;			// for simplifued
extern int noise_buf[];			// pink noise
extern uchar noise_off;			// generation
extern int seed;			// rand. seed

#ifdef WIN_AUDIO

extern HWAVEOUT aud_handle;
extern WAVEHDR **aud_head;
extern int aud_current;			// Current header
extern int aud_cnt;			// Number of headers in use
extern int aud_isopen;			// device was open
extern int winBufferSize;		// real buffer size
extern int winBufferCount;		// real buffer count

#endif

#ifdef MAC_AUDIO

extern char aud_buf[BUFFER_COUNT][];
extern int aud_rd;			// Next buffer to read out of list (to send to device)
extern int aud_wr;			// Next buffer to write.  aud_rd==aud_wr means empty buffer list
extern AudioDeviceID aud_dev;

#endif

//	WAV/OGG/MP3 input data buffering
extern int *inbuf;			// Buffer for input data (as 20-bit samples)
extern int ib_len;			// Length of input buffer (in ints)
extern volatile int ib_rd;		// Read-offset in inbuf
extern volatile int ib_wr;		// Write-offset in inbuf
extern volatile int ib_eof;		// End of file flag
extern volatile int ib_isrun;		// flag to exit the mix thread
extern int ib_cycle;			// Time in ms for a complete loop through the buffer
extern int (*ib_read)(int*, int);	// Routine to refill buffer
extern int ib_stream_type;		// type if mix input
// possible values of ib_stream_type:
#define	IBST_RAW	(0)		/* WAV/RAW */
#define	IBST_MP3	(1)		/* MP3 */
#define	IBST_OGG	(2)		/* OGG */

#ifdef WIN_MISC
extern HANDLE ib_hev;			// control event
extern HANDLE ib_het;			// read thread handle
extern CRITICAL_SECTION ib_cs;		// sync with inbuf thread
#endif

// Memory list stuff (1.4.4R+)
extern HeapList *heapRoot;		// 1-st memory list cell ("head")
extern HeapList *heapPtr;		// last memory list cell ("tail")
#if	DEBUG_MEM
extern FILE *dmem;			// log mem alloc/free
#endif

/*
 * The Functions Prototypes
 * --- --------- ----------
 */
// -- sbvars.c::
/* Clear all arrays (paranoja)
*/
void ClearStaticArrays(void);

// -- info.c::
/* print help and terminate
*/
void help(void);
/* print usage information and terminate
*/
void usage(void);
/* Error for bad p-drop args
*/
void bad_drop(void);
/* terminate, when bad slide found
*/
void bad_slide(void);
/* terminate, when bad sequence file
*/
void badSeq(void);
/* report about bad time line and terminate
*/
void badTime(char *tim);

// -- delay.c::
/* the simple delay
*/
void delay(int ms);

#ifdef WIN_MISC
/* the thread delay
*/
void delay_thr(int ms, HANDLE hEv);
#endif

// -- ibthr.c::
/* Start off the thread that fills the buffer
*/
void inbuf_start(int(*rout)(int*, int), int len);
/* cleanup inbuf thread (w/o input file) (wrong for U*X)
*/
void inbuf_stop(void);
/* The common read threed routine
*/
int THREAD inbuf_loop(void *dummy);
/* Read a chunk of int data from the input buffer.  This will
 *	always return enough data unless we have hit the end of the
 *	file, in which case it returns a lower number or 0.  If not
 *	enough data has been read by the input thread, then this
 *	thread pauses until data is ready -- but this should hopefully
 *	never happen.
 */
// TODO::Need more robast synch. with working thread!!
int inbuf_read(int *dst, int dlen);

// -- parsers.c::
/* Scan options.  Returns a flag indicating what is expected to
 * interpret the rest of the arguments: 0 normal, 'i' immediate
 * (-i option), 'p' -p option.
 */
int scanOptions(int *acp, char ***avp);
/* Handle an option string, breaking it into an (argc/argv) list
 * for scanOptions.
 */
void handleOptions(char *p);
/* Setup the ampadj[] array from the given -c spec-string
*/
void setupOptC(char *spec);
/* Read a line, discarding blank lines and comments.  Rets:
 * Another line?  Comments starting with '##' are displayed on
 * stderr.
 */
int readLine(void);
/* Get next word at '*lin', moving lin onwards, or return 0
*/
char *getWord(void);
/* Helper for 'Read a time-line of either type' --
 * -- Convenience for situations where buffer is being filled by
 * something other than readLine()
 */
void readTimeLine2(void);
/* Helper for 'Read a name definition' --
 * -- Convenience for situations where buffer is being filled by
 * something other than readLine()
 */
void readNameDef2(void);
/* generate a Name Definition
*/
void formatNameDef(char *fmt, ...);
/* generate a time line
*/
void formatTimeLine(int tim, char *fmt, ...);
/* Generate a list of Period structures, based on the tone-specs
 * passed in (ac,av)
 */
void readSeqImm(int ac, char **av);
/* Read a list of sequence files, and generate a list of Period
 * structures
 */
void readSeq(int ac, char **av);
/* Fill in all the correct information for the Periods, assuming
 * they have just been loaded using readTimeLine()
 */
void correctPeriods(void);
/* compare two Voices
*/
int voicesEq(Voice *v0, Voice *v1);
/* Read a name definition
*/
void readNameDef(void);
/* Read a time-line of either type
*/
void readTimeLine(void);
/* read time stamp
*/
int readTime(char *p, int *timp);	// Rets chars consumed, or 0 error
/* general pre-programmed routine
*/
void readPreProg(int ac, char **av);
/* Generate a p-drop sequence
 * Credits: Jonathan Bisson created the first version of this C
 * code.  This is a rewrite to make it fit with the rest of the
 * code better.
 */
void create_drop(int ac, char **av);
/* Generate a -p slide sequence
 * The idea of this is to hold the beat frequency constant, but
 * to slide down through the carrier frequencies from about 200Hz.
 *
 * -p slide [t<duration-minutes>] <carr>+<beat>/<amp> [extra tone-sets]
 */
void create_slide(int ac, char **av);

// -- wawraw.c::
/* If this is a WAV file we've been given, skip forward to the
 * 'data' section.  Don't bother checking any of the 'fmt '
 * stuff.  If they didn't give us a valid 16-bit stereo file at
 * the right rate, then tough!
 */
void find_wav_data_start(FILE *in);
/* Input raw audio data from the 'mix_in' stream, and convert to
 * 32-bit values (max 'dlen')
 */
int raw_mix_in(int *dst, int dlen);
/* Write a WAV header, and setup out_mode if byte-swapping is
 * required.  'byte_count' should have been set up by this point.
 */
void writeWAV(void);

// -- compite.c::
/* the sine table creation
*/
void init_sin_table(void);
/* Output a chunk of sound (a buffer-ful), then return
*/
void outChunk(void);
/* Calculate amplitude adjustment factor for frequency 'freq'
*/
double ampAdjust(double freq);
/* Correct channel values and types according to current period,
 * and current time
 */
void corrVal(int running);
/* Takes a set of points and repeats them twice, inverting the
 * second set, and then interpolates them using a periodic sinc
 * function (see http://www-ccrma.stanford.edu/~jos/resample/)
 * and writes them to arr[] in the same format as the sin_table[].
 */
void sinc_interpolate(double *dp, int np, int *arr);

// -- audio.c::
/* Setup audio device
*/
void setup_device(void);
#ifdef WIN_AUDIO
/* Audio callback for Win32
*/
void CALLBACK win32_audio_callback(HWAVEOUT hand, UINT uMsg,
	DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);
/* type debug info about WIN32 audio buffer status (not in use now)
*/
void debug_win32_buffer_status(void);
#endif
#ifdef MAC_AUDIO
/* Audio callback for Mac OS X
*/
OSStatus mac_callback(AudioDeviceID inDevice,
	const AudioTimeStamp *inNow,
	const AudioBufferList *inInputData,
	const AudioTimeStamp *inInputTime,
	AudioBufferList *outOutputData,
	const AudioTimeStamp *inOutputTime,
	void *inClientData);
#endif
/* write buffer to the audio device or output file
*/
void writeOut(char *buf, int siz);
/* cleanup audio output (handles only) (no code for MAC_AUDIO!)
*/
void audio_cleanup(void);

// -- timecalc.c::
/* calculate current time
*/
int calcNow(void);
/* synch. debug helper clock routine
*/
int userTime(void);

// -- consio.c::
/* Update a status line
*/
void status(char *err);
/* Display current period details
*/
void dispCurrPer(FILE *fp);

#ifndef SBG_GUI
/* print error message and terminate
*/
void error(char *fmt, ...);
#endif

/* print debug message
*/
void debug(char *fmt, ...);
/* print warning message
*/
void warn(char *fmt, ...);

// -- fmtmsg.c::
/* format the time
*/
int sprintTime(char *p, int tim);
/* format the Voice
*/
int sprintVoice(char *p, Voice *vp, Voice *dup);

// -- misc.c::
/* initialize memory allocator
*/
void InitAlloc(void);
/* allocate & clear memory with check (NON REENTRANT)
*/
void *Alloc(size_t len);
/* free memory, which allocated via Alloc (NON REENTRANT)
*/
void MFree(void *p);
/* free all the memory, allocated via Alloc
*/
void freeAll(void);
/* duplicate string with check
*/
char *StrDup(char *str);

// -- sbagenr.c::
/* The	M A I N  function
*/
#ifdef	SBG_GUI
int gui_main(int argc, char **argv);
#else
int main(int argc, char **argv);
#endif
/* The Main Loop
*/
void loop(void);
/* exit with full cleanup (may be wrong for U*X)
*/
void exitSBaGen(int code);

// OGG/MP3 functions
#ifdef OGG_DECODE

/* OGG init and start the inbuf thread for OGG stream
*/
void ogg_init(void);
/* OGG-related cleanup
*/
void ogg_term(void);

#endif

#ifdef MP3_DECODE

/* mad init and start the inbuf thread for MP3 stream
*/
void mp3_init(void);
/* mp3-related cleanup
*/
void mp3_term(void);

#endif

/* Unclassified macros
*/
#define ALLOC_ARR(cnt, type) ((type *)Alloc((cnt) * sizeof(type)))

#define NL "\n"

#define ST_AMP		(0x7FFFF)	// Amplitude of wave in sine-table
#define NS_ADJ		(12)		// Noise is generated internally with amplitude ST_AMP<<NS_ADJ
#define NS_BANDS	(9)		// Simpl. Pink Noise bands
#define NS_DITHER	(16)		// How many bits right to shift the noise for dithering
#define NS_AMP		(ST_AMP << NS_ADJ)
#define RAND_MULT	(75)		// rand. seq. parameter
#define ST_SIZ		(16384)		// Number of elements in sine-table (power of 2)
#define AMP_DA(pc)	(40.96 * (pc))	// Display value (%age) to ->amp value
#define AMP_AD(amp)	((amp) / 40.96)	// Amplitude value to display %age


/* Inline Functions
 * ------ ---------
 */
// Time-keeping functions
#define H24 (86400000)			// 24 hours
#define H12 (43200000)			// 12 hours

INLINE int t_per24(int t0, int t1)		// Length of period starting at t0, ending at t1.
{
 int td = t1 - t0;				// NB for t0==t1 this gives 24 hours, *NOT 0*
 return td > 0? td : td + H24;
}

INLINE int t_per0(int t0, int t1)		// Length of period starting at t0, ending at t1.
{
 int td = t1 - t0;				// NB for t0==t1 this gives 0 hours
 return td >= 0? td : td + H24;
}

INLINE int t_mid(int t0, int t1)		// Midpoint of period from t0 to t1
{
 return ((t1 < t0)? (H24 + t0 + t1) / 2 : (t0 + t1) / 2) % H24;
}

// Generate next sample for simulated pink noise, with same
// scaling as the sin_table[].  This version uses an inlined
// random number generator, and smooths the lower frequency bands
// as well.
//
INLINE int noise2(void)
{
 int tot;
 int off = nt_off++;
 int cnt = 1;
 Noise *ns = ntbl;
 Noise *ns1 = ntbl + NS_BANDS;

 tot = ((seed = seed * RAND_MULT % 131074) - 65535) *
	(NS_AMP / 65535 / (NS_BANDS + 1));

 while((cnt & off) && ns < ns1)
 {
  int val = ((seed = seed * RAND_MULT % 131074) - 65535) *
	(NS_AMP / 65535 / (NS_BANDS + 1));
  tot += ns -> val += ns -> inc = (val - ns -> val) / (cnt += cnt);
  ns++;
 }

 while(ns < ns1)
 {
  tot += (ns -> val += ns -> inc);
  ns++;
 }

 return noise_buf[noise_off++] = (tot >> NS_ADJ);
}

// round down to power of two

INLINE int dpwr2(int n)
{
 while(n & (n - 1))	// Make power of two
  n &= (n - 1);
 return n;
}

#ifdef __cplusplus
}
#endif

#endif	/* _INC_SBAGENR */

/* The END...
*/

