/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is graphical SBaGen sound channel control

#include "winsbagen.h"		// all the system and local inludes -- here

// we understood, that isChMute, isChMuteBak and chMuteId
// must be a struct members. But we don't want to make a new type
// for the whole project. So, let be separate arrays...
// (moreover, N_CH can be set to different value from the buttoms
// number!)

/* some globals
*/
volatile int isChMute[N_CH];	// real channel
HWND hwndDlgMute;		// Mute dialog

/* some locals
*/
static int isChMuteBak[N_CH];	// backup copy for "Cancel"

#define	N_CHBTN		(16)	/* only 16 buttons */
static int chMuteId[] =		// button ID's
{
 IDC_CH0,  IDC_CH1,  IDC_CH2,  IDC_CH3,
 IDC_CH4,  IDC_CH5,  IDC_CH6,  IDC_CH7,
 IDC_CH8,  IDC_CH9,  IDC_CH10, IDC_CH11,
 IDC_CH12, IDC_CH13, IDC_CH14, IDC_CH15
};

/* Muting initialization
*/
void ChmuteInit(void)
{
 // no window, all channels unmuted
 hwndDlgMute = NULL;
 memset((void *)isChMute, 0, sizeof(isChMute));
 memset(isChMuteBak, 0, sizeof(isChMuteBak));
}

/*
 * The Channel Mute Dialog
 * --- ------- ---- ------
 */
/* main dialog function
*/
BOOL CALLBACK Chmute_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam)
{
 BOOL fProcessed = TRUE;	//  WM_INITDIALOG   FALSE,
				//  . SetFocus
 switch(uMsg)			//  
 {
  HANDLE_MSG(hDlg, WM_INITDIALOG,	Chmute_Dlg_OnInitDialog);
  HANDLE_MSG(hDlg, WM_COMMAND,		Chmute_Dlg_OnCommand);
  default:
   fProcessed = FALSE;
 }
 return fProcessed;
}

/* dialog initialization
*/
BOOL Chmute_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP)
{
 int i;

 // make backup copy
 memcpy(isChMuteBak, (void *)isChMute, sizeof(isChMute));

 // controls -- to initial state
 for(i = 0; i < N_CHBTN; ++i)
 {
  if(i < N_CH)
   CheckDlgButton(hwnd, chMuteId[i],
	isChMute[i]? BST_CHECKED : BST_UNCHECKED);
  else
   ShowWindow(GetDlgItem(hwnd, chMuteId[i]), SW_HIDE);
 }

 return TRUE;
}

/* dialog commands
*/
void Chmute_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
 int i;

 switch(id)
 {
  // Channels:
#define	CH_MUBO(n)	\
	case IDC_CH##n:	\
         isChMute[n] = (IsDlgButtonChecked(hwnd, IDC_CH##n) == BST_CHECKED); \
	 break

  CH_MUBO(0);
  CH_MUBO(1);
  CH_MUBO(2);
  CH_MUBO(3);
  CH_MUBO(4);
  CH_MUBO(5);
  CH_MUBO(6);
  CH_MUBO(7);
  CH_MUBO(8);
  CH_MUBO(9);
  CH_MUBO(10);
  CH_MUBO(11);
  CH_MUBO(12);
  CH_MUBO(13);
  CH_MUBO(14);
  CH_MUBO(15);

#undef CH_MUBO

  // Exit
  case IDOK:			// accept
   ChmuteKill();
   break;

  case IDCANCEL:		// refuse
   // restore from backup
   for(i = 0; i < N_CH; ++i)	// memcpy() here not good, because
    isChMute[i] = isChMuteBak[i]; // we modify volatile variables
   ChmuteKill();
   break;
 }
}

/* dialog ending
*/
void ChmuteKill(void)
{
 if(hwndDlgMute)
 {
  DestroyWindow(hwndDlgMute);
  hwndDlgMute = NULL;
 }
}

/* The End...
*/

