/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is graphical SBaGen command line parser, dialog and it's thread function

#include "winsbagen.h"		// all the system and local inludes -- here

/* some globals
*/
char *fullCmdLine;		// raw command line
char *argvector[MAX_NARGV + 3];	// argvector build here
int argcounter;			// current argc
char progName[_MAX_PATH + 2];	// string with program name
char scriptName[_MAX_PATH + 2];	// full name of script
char mixName[_MAX_PATH + 2];	// full name of mix file
BOOL isMixRepeat;		// repeat flag for the mix file

/* some locals
*/
static char *tempMixName;	// the temporary name of mix file

/*
 * The Command Line Dialog
 * --- ------- ---- ------
 */
/* main dialog function
*/
BOOL CALLBACK Cmdline_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam)
{
 BOOL fProcessed = TRUE;	//  WM_INITDIALOG   FALSE,
				//  . SetFocus
 switch(uMsg)			//  
 {
  HANDLE_MSG(hDlg, WM_INITDIALOG,	Cmdline_Dlg_OnInitDialog);
  HANDLE_MSG(hDlg, WM_COMMAND,		Cmdline_Dlg_OnCommand);
  default:
   fProcessed = FALSE;
 }
 return fProcessed;
}

/* dialog initialization
*/
BOOL Cmdline_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP)
{
// controls -- to initial state
 if(fullCmdLine)
 {
  uniSetWindowText(hwnd, IDC_CMD, fullCmdLine);
 }

 tempMixName = cmdMalloc(_MAX_PATH + 2);
 strcpy(tempMixName, mixName);
 uniSetWindowText(hwnd, IDS_MIX_NAME, tempMixName);

 CheckDlgButton(hwnd, IDC_MIX_REP,
	isMixRepeat? BST_CHECKED : BST_UNCHECKED);

 return TRUE;
}

/* dialog commands
*/
void Cmdline_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
 int len;

 switch(id)
 {
  case IDB_MIX_OPEN:		// select file
   SelectFile(hwnd,
	_T("WAV sound\0*.wav\0RAW sound\0*.raw\0")
#ifdef OGG_DECODE
	_T("OGG sound\0*.ogg\0")
#endif
#ifdef MP3_DECODE
	_T("MP3 sound\0*.mp3\0")
#endif
	_T("\0"),
	_T("wav"),
	_T("Select Mix Background"),
	tempMixName);
   uniSetWindowText(hwnd, IDS_MIX_NAME, tempMixName);
   break;

  case IDB_MIX_CLOSE:		// cancel selected mix file
   tempMixName[0] = '\0';
   uniSetWindowText(hwnd, IDS_MIX_NAME, tempMixName);
   break;

  case IDOK:			// accept
   len = GetWindowTextLength(GetDlgItem(hwnd, IDC_CMD));
   if(fullCmdLine)
    free(fullCmdLine);
   fullCmdLine = cmdMalloc(len + 1);
#ifdef UNICODE
   {
    LPWSTR tsu = cmdMalloc((len + 1) * sizeof(WCHAR));

    GetWindowText(GetDlgItem(hwnd, IDC_CMD), tsu, len + 1);
    uni2asc(tsu, fullCmdLine);
    free(tsu);
   }
#else
   GetWindowText(GetDlgItem(hwnd, IDC_CMD), fullCmdLine, len + 1);
#endif

   strcpy(mixName, tempMixName);
   free(tempMixName);
   isMixRepeat = (IsDlgButtonChecked(hwnd, IDC_MIX_REP) == BST_CHECKED);

   EndDialog(hwnd, 0);
   break;

  case IDCANCEL:		// refuse
   free(tempMixName);
   EndDialog(hwnd, 0);
   break;
 }
}

/* universal 'SetWindowText()'
*/
void uniSetWindowText(HWND hwnd, int id, char *str)
{
#ifdef UNICODE
  LPWSTR tsu = asc2uni(str, NULL);

  SetWindowText(GetDlgItem(hwnd, id), tsu);
  free(tsu);
#else
  SetWindowText(GetDlgItem(hwnd, id), str);
#endif
}

/*
 * The Command Line Parser
 * --- ------- ---- ------
 */
/* get initial command line (if any) ind provide some initialization
*/
void InitCmd(void)
{
 TCHAR *cmd = GetCommandLine();
 int i;

 for(i = 0; i < MAX_NARGV; ++i)
  argvector[i] = NULL;

 scriptName[0] = '\0';
 mixName[0] = '\0';
 isMixRepeat = TRUE;		// the mix repeat bu default

 if(NULL == cmd || _T('\0') == *cmd)
 {
  strcpy(progName, PROG_IMAGE_NAME);
  fullCmdLine = NULL;
 }
 else
 {
  char *p, *pp = progName;
#ifdef UNICODE
  char *tsa = uni2asc(cmd, NULL);

  p = getParam(&pp, tsa);
#else
  p = getParam(&pp, cmd);
#endif
  if(*p)
  {
   fullCmdLine = cmdMalloc(strlen(p) + 1);
   strcpy(fullCmdLine, p);
  }
#ifdef UNICODE
  free(tsa);
#endif
 }
}

/* free the command line resources
*/
void FreeCmdMem(BOOL isFreeCmd)
{
 int i;

 for(i = 0; i < MAX_NARGV + 1; ++i)
 {
  if(argvector[i])
  {
   free(argvector[i]);
   argvector[i] = NULL;
  }
 }
 if(isFreeCmd && fullCmdLine)
 {
  free(fullCmdLine);
  fullCmdLine = NULL;
 }
}

/* create argvector
*/
void CreateArgv(void)
{
 FreeCmdMem(FALSE);

 // argv[0] -- program name
 argvector[0] = cmdMalloc(strlen(progName) + 1);
 strcpy(argvector[0], progName);
 argcounter = 1;

 // the second position(s) -- script (if any)
 if(mixName[0])
 {
  argvector[argcounter] = cmdMalloc(4); // -m+\0 == 4 bytes max.
  strcpy(argvector[argcounter++], isMixRepeat? "-m+" : "-m");
  argvector[argcounter] = cmdMalloc(strlen(mixName) + 1);
  strcpy(argvector[argcounter++], mixName);
 }

 // command line parameters (if any)
 if(fullCmdLine)
 {
  char *p = fullCmdLine;

  while(argcounter < MAX_NARGV)
  {
   p = getParam(&argvector[argcounter], p);
   if(NULL == argvector[argcounter])
    break;
   ++argcounter;
  }
 }

 // the last position -- script
 if(scriptName[0])
 {
  argvector[argcounter] = cmdMalloc(strlen(scriptName) + 1);
  strcpy(argvector[argcounter], scriptName);
  ++argcounter;
 }
}

/* get the next command line argument
*/
char *getParam(char **buff, char *line)
{
 int pass, mustAlloc = (NULL == *buff), cnt;
 char *p, *d;

 for(pass = 0; pass < 2; ++pass)
 {
  int isQ = 0;

  p = line;
  if(!pass)
   cnt = 0;
  else
  {
   if(mustAlloc)
    *buff = cmdMalloc(cnt + 1);
   d = *buff;
  }

  while(' ' == *p)
   ++p;

  while(*p)
  {
   if('\"' == *p)
   {
    isQ = !isQ;
    ++p;
    continue;
   }

   if(' ' == *p && !isQ)
    break;

   if(pass)
   {
    *d++ = *p++;
   }
   else
   {
    ++p;
    ++cnt;
   }
  }

 }

 // isQ may be true here, so we assume "\"xxx yyy zzz" as one big line
 // "xxx yyy zzz" w/o any diagnostics

 while(' ' == *p)
  ++p;

 // the parameter may be empty
 if(0 == cnt && mustAlloc)
 {
  free(*buff);
  *buff = NULL;
 }
 else
  (*buff)[cnt] = '\0';

 return p;
}

/* allocate memory for command line handle,
 * terminate, if bad
 */
void *cmdMalloc(size_t size)
{
 void *p = malloc(size);

 // this:
 // 1. seems unreachable;
 // 2. correct, because its call when the process has only
 // memory/graphical objects (no files, no threads)
 if(NULL == p)
 {
  MessageBox(NULL, _T("Can't handle command line"),
	_T("Hard Error"),
	MB_OK | MB_ICONHAND | MB_APPLMODAL);
  FreeCmdMem(TRUE);
  // do not any delete graphical objects (hfontLog, for example)
  exit(1);
 }

 return p;
}

/*
 * MISC. STUFF
 * ----- -----
 */
/* print current command line stuff
*/
void printCmdSt(void)
{
 static char fmtParm[] = "\'%s\'";

 if(scriptName[0])
 {
  log_printf("-- Current script file:");
  log_printf(fmtParm, scriptName);
 }
 else
 {
  log_printf("-- No current script file");
 }
 if(fullCmdLine)
 {
  if(*fullCmdLine)
  {
   log_printf("-- Current command line:");
   log_printf(fmtParm, fullCmdLine);
  }
  else
   log_printf("-- No current command line");
 }
 if(mixName[0])
 {
  log_printf("-- Current mix file:");
  log_printf(fmtParm, mixName);
 }
 log_printf("");
}

/* interractive select a file
*/
void SelectFile(HWND hwnd, LPTSTR filter, LPTSTR defExt, LPTSTR title, char *buf)
{
 OPENFILENAME ofn;
#ifdef UNICODE
 WCHAR tsu[_MAX_PATH + 2];
#endif

 ofn.lStructSize = sizeof(ofn);
 ofn.hwndOwner = hwnd;
 ofn.hInstance = hi;
 ofn.lpstrFilter = filter;
 ofn.lpstrCustomFilter = NULL;
 ofn.nMaxCustFilter = 0;
 ofn.nFilterIndex = 1;
#ifdef UNICODE
 asc2uni(buf, tsu);
 ofn.lpstrFile = tsu;
#else
 ofn.lpstrFile = buf;
#endif
 ofn.nMaxFile = _MAX_PATH;
 ofn.lpstrFileTitle = NULL;
 ofn.nMaxFileTitle = 0;
 ofn.lpstrInitialDir = NULL;
 ofn.lpstrTitle = title;
 ofn.Flags = OFN_EXPLORER | OFN_LONGNAMES | OFN_HIDEREADONLY |
	OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
 ofn.nFileOffset = 0;
 ofn.nFileExtension = 0;
 ofn.lpstrDefExt = defExt;
 ofn.lCustData = 0;
 ofn.lpfnHook = NULL;
 ofn.lpTemplateName = NULL;

 GetOpenFileName(&ofn);
#ifdef UNICODE
 uni2asc(tsu, buf);
#endif
}

/* startup of the SBG thread
*/
unsigned __stdcall sbgthrCaller(void *dummy)
{
 unsigned res = (unsigned)gui_main(argcounter, argvector);

 _endthreadex(res);

 return res;	// for C compiler
}

/* The End...
*/

