/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is graphical SBaGen startup and main window handle

#include "winsbagen.h"		// all the system and local inludes -- here

/* Global Variables
*/
// -- globals:
HINSTANCE hi;			// copy of application

HANDLE hthrUI;			// handle of user interface thread
DWORD idthrUI;			// id of user interface thread

HANDLE hthrSBG;			// handle of SBG thread
DWORD idthrSBG;			// id of SBG thread

HWND hwndApp;			// the main window
BOOL isRunDlgState;		// the main window status

char txtBuf[TXT_LINE_SIZE];	// buffer for the screen output
int cntTxtBuf;			// counter of chars in txt buffer
HFONT hfontLog;			// output font

// internal thread message queue
volatile QueueCell *tqeRoot;	// the last non-handled queue cell
volatile QueueCell *tqeLast;	// the last added queue cell
CRITICAL_SECTION tqeCS;		// lock for internal queue cell

#ifdef UNDER_CE
int isPortrait;			// screen orientation
#endif

// -- locals:
static HWND hwndLog;		// the log (list box)
static UINT timerId;		// the timer ID
#define	TIMER_ID	(1)	/* main window timer id */

#ifndef UNDER_CE
// the resizing stuff
static RECT rcCurrent;		// the courrent rectangle
static LONG minWidth;		// the minimal main window width
static LONG minHeigh;		// the minimal main window heigh
#endif


/* Main Function
*/
#ifdef UNDER_CE
int WINAPI WinMain(HINSTANCE hiExe, HINSTANCE hiPrev,
	LPWSTR lpszCmdLine, int nCmdShow)
#else
int WINAPI WinMain(HINSTANCE hiExe, HINSTANCE hiPrev,
	LPSTR lpszCmdLine, int nCmdShow)
#endif
{
 MSG msg;

// the one-time init...
 hi = hiExe;
 hthrUI = GetCurrentThread();
 idthrUI = GetCurrentThreadId();
 hthrSBG = NULL;
 isSbgRun = 0;
 isRunDlgState = 0;
 needNotifyExit = 0;
 txtBuf[cntTxtBuf = 0] = '\0';
 hwndLog = NULL;
 hfontLog = NULL;

#ifdef UNDER_CE
 isPortrait =
	(GetSystemMetrics(SM_CXSCREEN) < GetSystemMetrics(SM_CYSCREEN));
#endif

 // register the class of main window
 {
#ifndef UNDER_CE
  WNDCLASSEX wcl;

  // 32770 -- the std WIN32 dialog class id
  if(!GetClassInfoEx(hi, MAKEINTATOM(32770), &wcl))
   badWinInit();

  wcl.cbSize = sizeof(wcl);

#else
  WNDCLASS wcl;
  // L"Dialog" -- the std CE dialog class id
  if(!GetClassInfo(NULL, L"Dialog", &wcl))
   badWinInit();
#endif

  wcl.lpszClassName = _T(UNAME_MWND);
#ifndef UNDER_CE
  wcl.hIcon = (HICON)LoadImage(hi, MAKEINTRESOURCE(IDI_SBAGEN),
	IMAGE_ICON, 32, 32, 0);
  wcl.hIconSm = (HICON)LoadImage(hi, MAKEINTRESOURCE(IDI_SBAGEN),
	IMAGE_ICON, 16, 16, 0);
  if(!RegisterClassEx(&wcl))
#else
  wcl.hIcon = LoadIcon(hi, MAKEINTRESOURCE(IDI_SBAGEN));
  if(!RegisterClass(&wcl))
#endif
   badWinInit();
 }

 // check the single copy of appl.
 CreateEvent(NULL, FALSE, FALSE, _T(UNAME_RUN));
 // (we not need to story handle of this event --
 // the system MUST to do CloseHandle() for it
 // while the process will be terminated
 if(ERROR_ALREADY_EXISTS == GetLastError())
 {
  HWND oldHwnd;

  // already running -- bring it's window to top and die
  if(oldHwnd = FindWindow(_T(UNAME_MWND), NULL))
  {
   SetForegroundWindow((HWND)((ULONG)oldHwnd));
  }
  return 0;
 }

 InitCmd();
 ChmuteInit();
 TQueInit();

 {
  // create the monospace font for messages output
  LOGFONT lf;

  lf.lfHeight = 16;
  lf.lfWidth = 0;
  lf.lfEscapement = 0;
  lf.lfOrientation = 0;
  lf.lfItalic = FALSE;
  lf.lfUnderline = FALSE;
  lf.lfStrikeOut = FALSE;
  lf.lfCharSet = DEFAULT_CHARSET;
#ifndef UNDER_CE
  lf.lfOutPrecision = OUT_TT_PRECIS;
  lf.lfWeight = FW_REGULAR;
  lf.lfQuality = DEFAULT_QUALITY;
#else
  lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
  lf.lfWeight = FW_DONTCARE;
  lf.lfQuality = DRAFT_QUALITY;
#endif
  lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
  lf.lfPitchAndFamily = FIXED_PITCH;
  _tcscpy(lf.lfFaceName, _T("Courier New"));
  hfontLog = CreateFontIndirect(&lf);
 }

 // the main dialog -- non modal, because we need message loop
#ifndef UNDER_CE
 hwndApp = CreateDialog(hi, MAKEINTRESOURCE(IDD_SBAGEN), NULL, Sbagen_DlgProc);
#else
 hwndApp = CreateDialog(hi,
	MAKEINTRESOURCE(isPortrait? IDD_SBAGEN : IDD_SBAGEN_L),
	NULL, Sbagen_DlgProc);
#endif
 if(!hwndApp)
  badWinInit();

 // the message loop
 while(GetMessage(&msg, 0, 0, 0))
 {
  if(NULL == msg.hwnd)
  {
   // SbagenGthrHandleMessage(msg.message, msg.wParam, msg.lParam);
   // sorry.
   TQeHandleMessages();
   continue;
  }

  // handle main dialog messages
  if(IsDialogMessage(hwndApp, &msg)) continue;

  // if any non-modal dialog created -- handle it's massege here
  if(hwndDlgMute)
  {
   if(IsDialogMessage(hwndDlgMute, &msg)) continue;
  }

  TranslateMessage(&msg);
  DispatchMessage(&msg);
 }

 // aplication cleanup
 needNotifyExit = 0;
 isSbgRun = 0;
 if(hthrSBG)
 {
  WaitForSingleObject(hthrSBG, INFINITE);
  CloseHandle(hthrSBG);
  hthrSBG = NULL;
 }

 FreeCmdMem(TRUE);
 TQeDestroy();

 if(hfontLog)
  DeleteFont(hfontLog);

 return 0;
}

/* terminate by error while main window initialization
*/
void badWinInit(void)
{
 // this is fatal and, seems, ureachable error
 // so, all what we need -- terminate process
 // w/o free any resources
 MessageBox(NULL, _T("Can't Create main window"), _T("SBG Error"),
	MB_OK | MB_ICONHAND | MB_APPLMODAL);
 exit(1);
}

/*
 * Main dialog functions -- the main app. window
 */
/* main dialog function
*/
BOOL CALLBACK Sbagen_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam)
{
 BOOL fProcessed = TRUE;	//  WM_INITDIALOG   FALSE,
				//  . SetFocus
 switch(uMsg)			//  
 {
  HANDLE_MSG(hDlg, WM_INITDIALOG,	Sbagen_Dlg_OnInitDialog);
  HANDLE_MSG(hDlg, WM_COMMAND,		Sbagen_Dlg_OnCommand);
  HANDLE_MSG(hDlg, WM_TIMER,		Sbagen_Dlg_OnTimer);

#ifndef UNDER_CE
  case WM_SIZING:		// sizing window -- not supported by windowsx.h
   return Sbagen_Dlg_OnSizing(hDlg, (ULONG)wParam, (PRECT)lParam);
#endif

  default:
   fProcessed = FALSE;
 }
 return fProcessed;
}

/* dialog initialization
*/
BOOL Sbagen_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP)
{
 // hwndApp MAY NOT VALID HERE!! MUST BE USED hwnd

 // fill global variables
 hwndLog = GetDlgItem(hwnd, IDL_LOG);	// list == log

#ifndef UNDER_CE
 // get current rect
 GetWindowRect(hwnd, &rcCurrent);
 minWidth = rcCurrent.right - rcCurrent.left;	// width - 1
 minHeigh = rcCurrent.bottom - rcCurrent.top;	// heigh - 1
#endif

 // setup output
 if(hfontLog)
  SendMessage(hwndLog, WM_SETFONT, (WPARAM)hfontLog, MAKELPARAM(FALSE, 0));

 SendMessage(hwndLog,  LB_SETHORIZONTALEXTENT,
	(WPARAM)(TXT_LINE_SIZE * 6), (LPARAM)0);

 // controls -- to initial state
 SetupStopMainDlg(hwnd, FALSE);
 printCmdSt();

 // create a timer with int 1"
 timerId = SetTimer(hwnd, TIMER_ID, 1000, NULL);
 return TRUE;
}

/* dialog commands
*/
void Sbagen_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
 switch(id)
 {
// working
  case IDB_RUN:			// play the sound
   printCmdSt();
   CreateArgv();
   isSbgRun = 1;
   needNotifyExit = 1;
   hthrSBG = (HANDLE)_beginthreadex(NULL, 0,
	&sbgthrCaller, NULL, 0, &idthrSBG);
   if(NULL == hthrSBG)
   {
    isSbgRun = 0;
    MessageBox(hwnd, _T("Can't start SBaGen core"), _T("Run Error"), MB_OK);
   }
   else
    SetupRunMainDlg(hwnd);
   break;

  case IDB_STOP:		// stop the playing
   isSbgRun = 0;
   SetupWaitMainDlg(hwnd);
   break;

// parameters
  case IDB_OPEN:		// setup *.sbg file name
   SelectFile(hwnd,
#ifndef UNDER_CE
	_T("SBG script\0*.sbg\0Text file\0*.txt\0Any file\0*.*\0"),
#else
	_T("SBG script\0*.sbg\0Text file\0*.txt\0"),
#endif
	_T("sbg"),
	_T("Select SBG script"),
	scriptName);
   printCmdSt();
   break;

  case IDB_IMM:			// set immediate command line
   if(isRunDlgState)
   {
    if(!hwndDlgMute)
     hwndDlgMute = CreateDialog(hi, MAKEINTRESOURCE(IDD_CHMUTE),
	hwnd, Chmute_DlgProc);
    // if(hwndDlgMute)		// don't need here!! really!!
    //  SetFocus(hwnd);
   }
   else
   {
#ifndef UNDER_CE
    DialogBox(hi, MAKEINTRESOURCE(IDD_IMM), hwnd, Cmdline_DlgProc);
#else
    DialogBox(hi,
	MAKEINTRESOURCE(isPortrait? IDD_IMM : IDD_IMM_L),
	hwnd, Cmdline_DlgProc);
#endif
    printCmdSt();
   }
   break;

  case IDB_CLOSE:		// purge current *.sbg file name
   scriptName[0] = '\0';
   printCmdSt();
   break;

// termination
  case IDCANCEL:		// exit program -- free all
  case IDOK:			// -"-
   SetupWaitMainDlg(hwnd);
   Sbagen_Dlg_Terminate(hwnd);
   break;
 }
}

/* timer handling
*/
void Sbagen_Dlg_OnTimer(HWND hwnd, UINT id)
{
// this is a "patch" routine, it need only for VERY rerely
// case when GUI thr lost message (XM_*) from SBG thread
 TQeHandleMessages();
}

#ifndef UNDER_CE
/* sizing main window
*/
BOOL Sbagen_Dlg_OnSizing(HWND hwnd, ULONG nEdge, RECT *rc)
{
 LONG dX, dY;
 RECT rcT;
 HWND hwndT;

 // prevent to make window too small
 if(rc -> right - rc -> left < minWidth)
 {
  if(rc -> left == rcCurrent.left)
   rc -> right = rcCurrent.left + minWidth;
  else
  {
   if(rc -> right == rcCurrent.right)
   {
    rc -> left = rcCurrent.right - minWidth;
   }
   else
   {
    rc -> right = (rc -> left = rcCurrent.left) + minWidth;
   }
  }
 }
 if(rc -> bottom - rc -> top < minHeigh)
 {
  if(rc -> top == rcCurrent.top)
   rc -> bottom = rcCurrent.top + minHeigh;
  else
  {
   if(rc -> bottom == rcCurrent.bottom)
   {
    rc -> top = rcCurrent.bottom - minHeigh;
   }
   else
   {
    rc -> bottom = (rc -> top = rcCurrent.top) + minHeigh;
   }
  }
 }
 // get the changes
 dX = (rc -> right - rc -> left) - (rcCurrent.right - rcCurrent.left);
 dY = (rc -> bottom - rc -> top) - (rcCurrent.bottom - rcCurrent.top);
 // save new window rect
 CopyRect(&rcCurrent, rc);

 // AJUST CONTROL'S SIZES/POSITION
 // -- list (output) control
 GetWindowRect(hwndT = GetDlgItem(hwnd, IDL_LOG), &rcT);
 SetWindowPos(hwndT, hwnd, 0, 0,
	(rcT.right - rcT.left) + dX,
	(rcT.bottom - rcT.top) + dY,
	SWP_NOMOVE | SWP_NOZORDER | SWP_NOOWNERZORDER);

 // -- Exit Button position
 GetWindowRect(hwndT = GetDlgItem(hwnd, IDCANCEL), &rcT);
 ScreenToClient(hwnd, (PPOINT)(&rcT));
 SetWindowPos(hwndT, hwnd,
	rcT.left + dX,
	rcT.top,
	0, 0,
	SWP_NOSIZE | SWP_NOZORDER | SWP_NOOWNERZORDER);

 return TRUE;
}
#endif


/* the main window termination
*/
void Sbagen_Dlg_Terminate(HWND hwnd)
{
 // no need to output
 // (if the main thr accepted any thread messages, before
 // WM_QUIT, it will be handled quiet)
 hwndLog = NULL;
 // clean up:
 needNotifyExit = 0;
 isSbgRun = 0;
 if(hthrSBG)
 {
  WaitForSingleObject(hthrSBG, INFINITE);
  CloseHandle(hthrSBG);
  hthrSBG = NULL;
 }

 // kill the timer
 if(timerId)
  KillTimer(hwnd, TIMER_ID);

 // complete:
 DestroyWindow(hwnd);
 PostQuitMessage(0);
}

/*
 * Setup the controls
 */
/* setup the main window to the stop state
*/
void SetupStopMainDlg(HWND hwnd, BOOL wasError)
{
 EnDis(hwnd, IDB_RUN,   TRUE);
 EnDis(hwnd, IDB_STOP,  FALSE);
 SetFocus(GetDlgItem(hwnd, IDB_RUN));

 EnDis(hwnd, IDB_OPEN,  TRUE);
 EnDis(hwnd, IDB_CLOSE, TRUE);

 EnDis(hwnd, IDB_IMM,   TRUE);
 SetWindowText(GetDlgItem(hwnd, IDB_IMM), _T("++"));

 EnDis(hwnd, IDCANCEL,  TRUE);
 SetWindowText(hwnd, wasError? _T("SBaGen -- Stop/Error") :
	_T("SBaGen -- Stoped"));

 if(hwndDlgMute)
  ChmuteKill();

 isRunDlgState = FALSE;
}

/* setup the main window to the running state
*/
void SetupRunMainDlg(HWND hwnd)
{
 EnDis(hwnd, IDB_RUN,   FALSE);
 EnDis(hwnd, IDB_STOP,  TRUE);
 SetFocus(GetDlgItem(hwnd, IDB_STOP));

 EnDis(hwnd, IDB_OPEN,  FALSE);
 EnDis(hwnd, IDB_CLOSE, FALSE);

 EnDis(hwnd, IDB_IMM,   TRUE);
 SetWindowText(GetDlgItem(hwnd, IDB_IMM), _T("--"));

 EnDis(hwnd, IDCANCEL,  TRUE);
 SetWindowText(hwnd, _T("SBaGen -- Playing"));

 isRunDlgState = TRUE;
}

/* setup the main window to the waiting state
*/
void SetupWaitMainDlg(HWND hwnd)
{
 EnDis(hwnd, IDB_RUN,   FALSE);
 EnDis(hwnd, IDB_STOP,  FALSE);
 SetFocus(GetDlgItem(hwnd, IDB_RUN));

 EnDis(hwnd, IDB_OPEN,  FALSE);
 EnDis(hwnd, IDB_CLOSE, FALSE);
 EnDis(hwnd, IDB_IMM,   FALSE);
 EnDis(hwnd, IDCANCEL,  FALSE);
 SetWindowText(hwnd, _T("SBaGen -- Waiting"));
}

/*
 * Text output functions
 */
/* simple 'printf' for main thread (can be used only while
 * working thread(s) not ruunning
 */
void log_printf(char *fmt, ...)
{
 va_list ap;
 int olen;

 va_start(ap, fmt);
 olen = _vsnprintf(txtBuf, TXT_LINE_SIZE, fmt, ap);
 if(olen < 0)
  txtBuf[TXT_LINE_SIZE - 1] = '\0';
 log_puts(txtBuf, TRUE);
}

/* log string to a log window (MAIN thread)
*/
void log_puts(char *pszLogLine, BOOL isLineFeed)
{
#ifdef UNICODE
 WCHAR convBuf[TXT_LINE_SIZE];
#endif
 static int ixLast = -1;
 static BOOL wasCR = FALSE;

 if(NULL == hwndLog)				// paranoja
  return;

 if(wasCR && ixLast != -1)			// replace the last string
  ListBox_DeleteString(hwndLog, ixLast);	// errors impossible

 // remove old lines, if need
 if(ixLast > LOG_MAX_MAX_NLINES)
  while(ixLast > LOG_MAX_NLINES)
  {
   ListBox_DeleteString(hwndLog, 0);
   --ixLast;
  }

#ifdef UNICODE
 asc2uni(pszLogLine, convBuf);
#endif

 for(;;)
 {
#ifdef UNICODE
  ixLast = ListBox_InsertString(hwndLog, ixLast, convBuf);
#else
  ixLast = ListBox_InsertString(hwndLog, ixLast, pszLogLine);
#endif
  if(ixLast != LB_ERRSPACE && ixLast != LB_ERR)
  {
   ListBox_SetCurSel(hwndLog, ixLast);
   break;
  }
  ListBox_DeleteString(hwndLog, 0);
 }

 if(isLineFeed)		// "...\n"
 {
  wasCR = FALSE;
  ++ixLast;
 }
 else
  wasCR = TRUE;
}

/* non-window SBaGen GUI-thread message(s) handler
*/
void SbagenGthrHandleMessage(UINT message, WPARAM wp, LPARAM lp)
{
 static BOOL wasError = FALSE;

 switch(message)
 {
  case XM_STDIO:	// any stdio function
   wasError = (0 != (wp & SCR_ERROR));
   stdio_output((ULONG)wp, lp);
   break;

  case XM_SBEXIT:	// SBG thr exit notification
   isSbgRun = 0;
   if(hthrSBG)
   {
    WaitForSingleObject(hthrSBG, INFINITE);
    CloseHandle(hthrSBG);
    hthrSBG = NULL;
   }
   if(cntTxtBuf)
   {
    log_puts(txtBuf, TRUE);
    txtBuf[cntTxtBuf = 0] = '\0';
   }
   log_puts(txtBuf, TRUE);	// extra '\n'
   SetupStopMainDlg(hwndApp, wasError);
   break;

  default:
   // nothing to do
   break;
 }
}

/* the stdio output -- complemental part of the gui_stdio.c functions
*/
void stdio_output(ULONG flags, LPARAM lp)
{
 char *str, *p;

 if(flags & SCR_FFUSH)		// ffush function
 {
  if(cntTxtBuf)
  {
   log_puts(txtBuf, FALSE);	// like a \r, *BUT* the string may be continued
  }
  return;
 }

 if(flags & SCR_SINGLE)		// putc function
 {
  char c = (char)lp;

  if(0 == (flags & SCR_NOEOL))
  {
   log_puts(txtBuf, '\n' == c);
   txtBuf[cntTxtBuf = 0] = '\0';
  }
  else
  {
   // while we use 'putc', we do output as a correct TTY, whitch carry
   // too wide strings to the next line
   if(c == '\t')
   {
    stdio_handle_tab();
    return;
   }
   if(cntTxtBuf >= TXT_LINE_SIZE - 1)
   {
    log_puts(txtBuf, TRUE);
    cntTxtBuf = 0;
   }
   txtBuf[cntTxtBuf++] = c;
   txtBuf[cntTxtBuf] = '\0';
  }
  return;
 }				// -- putc

 // here we have a string, after we done we *MUST* to free() its memory
 p = str = (char *)lp;
 while(*p)
 {
  switch(*p)
  {
   case '\n':
    ++p;
    log_puts(txtBuf, TRUE);
    txtBuf[cntTxtBuf = 0] = '\0';
    break;

   case '\r':			// noncence in the string body -- ignore
    ++p;
    continue;

   case '\t':			// h-tab -- convert to blanks
    stdio_handle_tab();
    ++p;
    break;

   default:			// plain char
    // we do output as a correct TTY, whitch carry
    // too wide strings to the next line
    if(cntTxtBuf >= TXT_LINE_SIZE - 1)
    {
     log_puts(txtBuf, TRUE);
     cntTxtBuf = 0;
    }
    txtBuf[cntTxtBuf++] = *p++;
    txtBuf[cntTxtBuf] = '\0';
    break;
  }
 }				// for *p;
 if(0 == (flags & SCR_NOEOL))
 {
  log_puts(txtBuf, 0 != (flags & SCR_LF));
  txtBuf[cntTxtBuf = 0] = '\0';
 }

 // done!
 free(str);
}

/* horisontal tab ('\t') handling
*/
void stdio_handle_tab(void)
{
 int tcnt = TAB_SIZE - cntTxtBuf % TAB_SIZE;

 if(cntTxtBuf + tcnt >= TXT_LINE_SIZE - 2)
 {
  // extremal case -- tab out of line == '\n'
  log_puts(txtBuf, TRUE);
  txtBuf[cntTxtBuf = 0] = '\0';
 }
 else
 {
  while(tcnt)
  {
   txtBuf[cntTxtBuf++] = ' ';
   --tcnt;
  }
  txtBuf[cntTxtBuf] = '\0';
 }
}

/*
 * Own thread message queue stuff
 * --- ------ ------- ----- -----
 */
// NOTE:: this is only a path, it need, because windows can lost
// messages from the PostThreadMessage() call

/* initialize thread message
*/
void TQueInit(void)
{
 tqeRoot = tqeLast = NULL;
 InitializeCriticalSection(&tqeCS);
}

/* post thread message via own queue
*/
void TQuePostMessage(UINT message, WPARAM wp, LPARAM lp)
{
 QueueCell *qc;

 if(NULL == (qc = malloc(sizeof(QueueCell))))	// oops...
 {						// seems, unreachable
  MessageBox(NULL, _T("No free memory at all"), _T("SBG FATAL"),
	MB_OK | MB_ICONHAND | MB_APPLMODAL);
  // don't care about cleanup -- it's unreacable!!
  exit(1);
 }

 // ok, lets go
 qc -> message = message;
 qc -> wp = wp;
 qc -> lp = lp;
 qc -> nxt = NULL;

 EnterCriticalSection(&tqeCS);
 if(NULL == tqeLast)
 {
  tqeLast = tqeRoot = qc;
 }
 else
 {
  tqeLast -> nxt = qc;
  tqeLast = qc;
 }
 LeaveCriticalSection(&tqeCS);

 // ok, our message can be handled BEFORE we post it!
 PostThreadMessage(idthrUI, message, wp, lp);
}

/* handle current message list
*/
void TQeHandleMessages(void)
{
 QueueCell *qc;

 for(;;)
 {
  EnterCriticalSection(&tqeCS);
  if(NULL == tqeRoot)
  {
   LeaveCriticalSection(&tqeCS);
   return;
  }
  else
  {
   qc = (QueueCell *)tqeRoot;
   if(NULL == (tqeRoot = qc -> nxt))
    tqeLast = NULL;
  }
  LeaveCriticalSection(&tqeCS);

  SbagenGthrHandleMessage(qc -> message, qc -> wp, qc -> lp);

  free(qc);
 }
}

/* destroy message queue (all treads halted)
*/
void TQeDestroy(void)
{
 // the bullshit::
 while(tqeRoot)			// it must be unreachable,
 {				// or some memory still lost
  QueueCell *qc = (QueueCell *)tqeRoot;
  tqeRoot = qc -> nxt;
  free(qc);
 }
 tqeLast = NULL;

 // the single right thing to do here:
 DeleteCriticalSection(&tqeCS);
}

/* the end...
*/

