/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is main header for the graphical SBaGen Windows stuff

/* The Includes::
 * --- ----------
 */
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#ifndef UNDER_CE
#include <process.h>
#endif


#include "gui_stdio.h"		// stdio API replacement

#include "../sbagenr.h"		// it can include all the oter files, or not

#ifndef UNDER_CE
#include "resource.h"
#else
#include "../sbagene/resource.h"
#endif

/* Parameters
 * ----------
 */
// max. screen line lenght
#define	TXT_LINE_SIZE		(512)	/* seems enougt */

// max screen heigh
#define	LOG_MAX_MAX_NLINES	(512)	/* upper bound to run cleanup */
#define	LOG_MAX_NLINES		(LOG_MAX_MAX_NLINES - 32) /* after cleanup */

// command line parameters
#define	MAX_NARGV		(64)	/* seems enough */

// the default program image name
#ifndef	UNICODE
#define	PROG_IMAGE_NAME		".\\sbagenw.exe"
#else
#ifndef UNDER_CE
#define	PROG_IMAGE_NAME		".\\sbagenwu.exe"
#else
#define	PROG_IMAGE_NAME		".\\sbagene.exe"
#endif
#endif

// the unique name for single instance
#define	UNAME_RUN		"SBaGenRWUCE"
// the name of main window class
#define	UNAME_MWND		"SBaGenWndCls"

/*
 * GUI internal type(s)
 * --- -------- -------
 */
// own thread queue element
typedef struct tagQueueCell
{
 volatile struct tagQueueCell *nxt; // link to next cell
 UINT message;			// similar to MSG.message
 WPARAM wp;			// similar to MSG.wParam
 LPARAM lp;			// similar to MSG.lParam
} QueueCell;

/* Useful macros::
 * ------ --------
 */
// enable/disable a control
#define	EnDis(hw, id, qen)	\
	EnableWindow(GetDlgItem((hw), (id)), (qen))


/*
 * Windows functions and variables::
 * ------- --------- --- -----------
 */
// sbagenw.c::
// -- variables:
extern HINSTANCE hi;			// copy of application
extern HANDLE hthrUI;			// handle of user interface thread
extern DWORD idthrUI;			// id of user interface thread
extern HANDLE hthrSBG;			// handle of SBG thread
extern DWORD idthrSBG;			// id of SBG thread
extern HWND hwndApp;			// the main window
extern char txtBuf[];			// buffer for the screen output
extern int cntTxtBuf;			// counter of chars in txt buffer
extern HFONT hfontLog;			// output font
extern volatile QueueCell *tqeRoot;	// the last non-handled queue cell
extern volatile QueueCell *tqeLast;	// the last added queue cell
extern CRITICAL_SECTION tqeCS;		// lock for internal queue cell
#ifdef UNDER_CE
extern int isPortrait;			// screen orientation
#endif
/* Main Function
*/
#ifdef UNDER_CE
int WINAPI WinMain(HINSTANCE hiExe, HINSTANCE hiPrev,
	LPWSTR lpszCmdLine, int nCmdShow);
#else
int WINAPI WinMain(HINSTANCE hiExe, HINSTANCE hiPrev,
	LPSTR lpszCmdLine, int nCmdShow);
#endif
/* terminate by error while main window initialization
*/
void badWinInit(void);
/* main dialog function
*/
BOOL CALLBACK Sbagen_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam);
/* dialog initialization
*/
BOOL Sbagen_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP);
/* dialog commands
*/
void Sbagen_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
/* timer handling
*/
void Sbagen_Dlg_OnTimer(HWND hwnd, UINT id);

#ifndef UNDER_CE
/* sizing main window
*/
BOOL Sbagen_Dlg_OnSizing(HWND hDlg, ULONG nEdge, RECT *rc);
#endif

/* the main window termination
*/
void Sbagen_Dlg_Terminate(HWND hwnd);
/* setup the main window to the stop state
*/
void SetupStopMainDlg(HWND hwnd, BOOL wasError);
/* setup the main window to the running state
*/
void SetupRunMainDlg(HWND hwnd);
/* setup the main window to the waiting state
*/
void SetupWaitMainDlg(HWND hwnd);
/* simple 'printf' for main thread (can be used only while
 * working thread(s) not ruunning
 */
void log_printf(char *fmt, ...);
/* log string to a log window (MAIN thread)
*/
void log_puts(char *pszLogLine, BOOL isLineFeed);
/* non-window SBaGen GUI-thread message(s) handler
*/
void SbagenGthrHandleMessage(UINT message, WPARAM wp, LPARAM lp);
/* the stdio output -- complemental part of the gui_stdio.c functions
*/
void stdio_output(ULONG flags, LPARAM lp);
/* horisontal tab ('\t') handling
*/
void stdio_handle_tab(void);

/* initialize thread message
*/
void TQueInit(void);
/* post thread message via own queue
*/
void TQuePostMessage(UINT message, WPARAM wp, LPARAM lp);
/* handle current message list
*/
void TQeHandleMessages(void);
/* destroy message queue (all treads halted)
*/
void TQeDestroy(void);

// gui_cmd.c::
// -- variables:
extern char *fullCmdLine;		// raw command line
extern char *argvector[];		// argvector build here
extern int argcounter;			// current argc
extern char progName[];			// string with program name
extern char scriptName[];		// full name of script
char mixName[];				// full name of mix file
BOOL isMixRepeat;			// repeat flag for the mix file

/* main dialog function
*/
BOOL CALLBACK Cmdline_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam);
/* dialog initialization
*/
BOOL Cmdline_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP);
/* dialog commands
*/
void Cmdline_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
/* universal 'SetWindowText()'
*/
void uniSetWindowText(HWND hwnd, int id, char *str);
/* get initial command line (if any) ind provide some initialization
*/
void InitCmd(void);
/* free the command line resources
*/
void FreeCmdMem(BOOL isFreeCmd);
/* create argvector
*/
void CreateArgv(void);
/* get the next command line argument
*/
char *getParam(char **buff, char *line);
/* allocate memory for command line handle,
 * terminate, if bad
 */
void *cmdMalloc(size_t size);
/* print current command line stuff
*/
void printCmdSt(void);
/* interractive select a file
*/
void SelectFile(HWND hwnd, LPTSTR filter, LPTSTR defExt, LPTSTR title, char *buf);
/* startup of the SBG thread
*/
unsigned __stdcall sbgthrCaller(void *dummy);

// gui_chctl.c::
// -- variables:
// [see also gui_cmd.c]
extern HWND hwndDlgMute;	// Mute dialog

/* Muting initialization
*/
void ChmuteInit(void);
/* main dialog function
*/
BOOL CALLBACK Chmute_DlgProc(HWND hDlg, UINT uMsg,
	WPARAM wParam, LPARAM lParam);
/* dialog initialization
*/
BOOL Chmute_Dlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lP);
/* dialog commands
*/
void Chmute_Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
/* dialog ending
*/
void ChmuteKill(void);


/* the end...
*/

