/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen VARIABLES file

#include "sbagenr.h"

int *sin_table = NULL;

int *waves[100];		// Pointers are either 0 or point to a sin_table[]-style array of int

Channel chan[N_CH];		// Current channel states
int now = 0;			// Current time (milliseconds from midnight)
Period *per = NULL;		// Current period
NameDef *nlist =  NULL;		// Full list of name definitions

int *tmp_buf = NULL;		// Temporary buffer for 20-bit mix values
short *out_buf = NULL;		// Output buffer
int out_bsiz = 0;		// Output buffer size (bytes)
int out_blen = 0;		// Output buffer length (samples) (1.0* or 0.5* out_bsiz)
int out_bps = 0;		// Output bytes per sample (2 or 4)
int out_buf_ms = 0;		// Time to output a buffer-ful in ms
int out_buf_lo = 0;		// Time to output a buffer-ful, fine-tuning in ms/0x10000
int out_fd = -1;		// Output file descriptor
FILE *out_file = NULL;		// FILE for out_fd
int out_rate = 44100;		// Sample rate
int out_rate_def = 1;		// Sample rate is default value, not set by user
int out_mode = 1;		// Output mode: 0 unsigned char[2], 1 short[2], 2 swapped short[2]
int out_prate = 10;		// Rate of parameter change (for file and pipe output only)
int fade_int = 60000;		// Fade interval (ms)
FILE *in = NULL;		// Input sequence file
int in_lin = 0;			// Current input line
char buf[LBUF_SIZE];		// Buffer for current line
char buf_copy[LBUF_SIZE];	// Used to keep unmodified copy of line
char *lin = NULL;		// Input line (uses buf[])
char *lin_copy = NULL;		// Copy of input line
double spin_carr_max = 0.0;	// Maximum 'carrier' value for spin (really max width in us)

int fast_tim0 = -1;		// First time mentioned in the sequence file (for -q and -S option)
int fast_tim1 = -1;		// Last time mentioned in the sequence file (for -E option)
int fast_mult = 0;		// 0 to sync to clock (adjusting as necessary), or else sync to
				//  output rate, with the multiplier indicated
S64 byte_count = -1;		// Number of bytes left to output, or -1 if unlimited
int tty_erase = 0;		// Chars to erase from current line (for ESC[K emulation)

int opt_D = 0;
int opt_M = 0;
int opt_Q = 0;
int opt_S = 0;
int opt_E = 0;
int opt_W = 0;
int opt_O = 0;
int opt_L = -1;			// Length in ms, or -1
int opt_T = -1;			// Start time in ms, or -1
char *opt_o = NULL;		// File name to output to, or 0
char *opt_m = NULL;		// File name to read mix data from, or 0
int opt_mrep = 0;		// repeat wav/raw file
char *opt_d = "/dev/dsp";	// Output device

FILE *mix_in = NULL;		// Input stream for mix sound data, or 0
int mix_cnt = -1;		// Version number from mix filename (#<digits>), or -1
int bigendian = 0;		// Is this platform Big-endian?
int mix_flag = 0;		// Has 'mix/*' been used in the sequence?
long raw_init_pos = 0;		// init position of raw/wav stream
int wav_data_len = 0;		// lenght of wav data chunk
int wav_data_cnt = 0;		// counter of wav data
int opt_c = 0;			// Number of -c option points provided (max 16)

AmpAdj ampadj[AMP_ADJ_SIZ];	// List of maximum 16 (freq,adj) pairs, freq-increasing order

char *pdir = NULL;		// Program directory (used as second place to look for -m files)

// Noise stuff
Noise ntbl[NS_BANDS];		// variables
int nt_off = 0;			// for simplifued
int noise_buf[256];		// pink noise
uchar noise_off = 0;		// generation
int seed = 2;			// rand. seed

#ifdef WIN_AUDIO

HWAVEOUT aud_handle = 0;
WAVEHDR **aud_head = NULL;
int aud_current = 0;		// Current header
int aud_cnt = 0;		// Number of headers in use
int aud_isopen = 0;		// device was open
int winBufferSize = BUFFER_SIZE;	// real buffer size
int winBufferCount = BUFFER_COUNT;	// real buffer count

#endif

#ifdef MAC_AUDIO

char aud_buf[BUFFER_COUNT][BUFFER_SIZE];
int aud_rd = 0;			// Next buffer to read out of list (to send to device)
int aud_wr = 0;			// Next buffer to write.  aud_rd==aud_wr means empty buffer list
AudioDeviceID aud_dev;

#endif

//
//	WAV/OGG/MP3 input data buffering
//

int *inbuf = NULL;		// Buffer for input data (as 20-bit samples)
int ib_len = 0;			// Length of input buffer (in ints)
volatile int ib_rd = 0;		// Read-offset in inbuf
volatile int ib_wr = 0;		// Write-offset in inbuf
volatile int ib_eof = 0;	// End of file flag
volatile int ib_isrun = 0;	// flag to exit the mix thread
int ib_cycle = 100;		// Time in ms for a complete loop through the buffer
int (*ib_read)(int*, int) = NULL;  // Routine to refill buffer
int ib_stream_type = IBST_RAW;	// type if mix input
#ifdef WIN_MISC
HANDLE ib_hev = NULL;		// control event
HANDLE ib_het = NULL;		// read thread handle
CRITICAL_SECTION ib_cs;		// sync with inbuf thread
#endif

// Memory list stuff (1.4.4R+)
HeapList *heapRoot = NULL;	// 1-st memory list cell ("head")
HeapList *heapPtr = NULL;	// last memory list cell ("tail")
#if	DEBUG_MEM
FILE *dmem = NULL;		// log mem alloc/free
#endif

/* Clear all arrays (paranoja)
*/
void ClearStaticArrays(void)
{
#define CZ(v)	memset((v), 0, sizeof(v))

 CZ(waves);
 CZ(chan);
 CZ(buf);
 CZ(buf_copy);
 CZ(ampadj);

 CZ(ntbl);
 CZ(noise_buf);

#ifdef MAC_AUDIO
 CZ(aud_buf);
#endif

#undef CZ

 // ...and scalar variables too...
 sin_table = NULL;

 now = 0;			// Current time (milliseconds from midnight)
 per = NULL;			// Current period
 nlist = NULL;			// Full list of name definitions

 tmp_buf = NULL;		// Temporary buffer for 20-bit mix values
 out_buf = NULL;		// Output buffer
 out_bsiz = 0;			// Output buffer size (bytes)
 out_blen = 0;			// Output buffer length (samples) (1.0* or 0.5* out_bsiz)
 out_bps = 0;			// Output bytes per sample (2 or 4)
 out_buf_ms = 0;		// Time to output a buffer-ful in ms
 out_buf_lo = 0;		// Time to output a buffer-ful, fine-tuning in ms/0x10000
 out_fd = -1;			// Output file descriptor
 out_file = NULL;		// FILE for out_fd
 out_rate = 44100;		// Sample rate
 out_rate_def = 1;		// Sample rate is default value, not set by user
 out_mode = 1;			// Output mode: 0 unsigned char[2], 1 short[2], 2 swapped short[2]
 out_prate = 10;		// Rate of parameter change (for file and pipe output only)
 fade_int = 60000;		// Fade interval (ms)
 in = NULL;			// Input sequence file
 in_lin = 0;			// Current input line
 lin = NULL;			// Input line (uses buf[])
 lin_copy = NULL;		// Copy of input line
 spin_carr_max = 0.0;		// Maximum 'carrier' value for spin (really max width in us)

 fast_tim0 = -1;		// First time mentioned in the sequence file (for -q and -S option)
 fast_tim1 = -1;		// Last time mentioned in the sequence file (for -E option)
 fast_mult = 0;			// 0 to sync to clock (adjusting as necessary), or else sync to
				//  output rate, with the multiplier indicated
 byte_count = -1;		// Number of bytes left to output, or -1 if unlimited
 tty_erase  = 0;		// Chars to erase from current line (for ESC[K emulation)

 opt_D = 0;
 opt_M = 0;
 opt_Q = 0;
 opt_S = 0;
 opt_E = 0;
 opt_W = 0;
 opt_O = 0;
 opt_L = -1;			// Length in ms, or -1
 opt_T = -1;			// Start time in ms, or -1
 opt_o = NULL;			// File name to output to, or 0
 opt_m = NULL;			// File name to read mix data from, or 0
 opt_mrep = 0;			// repeat wav/raw flag
 opt_d = "/dev/dsp";		// Output device

 mix_in = NULL;			// Input stream for mix sound data, or 0
 mix_cnt = -1;			// Version number from mix filename (#<digits>), or -1
 bigendian = 0;			// Is this platform Big-endian?
 mix_flag = 0;			// Has 'mix/*' been used in the sequence?
 raw_init_pos = 0;		// init position of raw/wav stream
 wav_data_len = 0;		// lenght of wav data chunk
 wav_data_cnt = 0;		// counter of wav data

 opt_c = 0;			// Number of -c option points provided (max 16)

 pdir = NULL;			// Program directory (used as second place to look for -m files)

 nt_off = 0;			// for simplifued
 noise_off = 0;			// generation
 seed = 2;			// rand. seed

#ifdef WIN_AUDIO

 aud_head = NULL;
 aud_handle = 0;
 aud_current = 0;		// Current header
 aud_cnt = 0;			// Number of headers in use
 aud_isopen = 0;		// device was open
 winBufferSize = BUFFER_SIZE;	// real buffer size
 winBufferCount = BUFFER_COUNT;	// real buffer count

#endif

#ifdef MAC_AUDIO

 aud_rd = 0;			// Next buffer to read out of list (to send to device)
 aud_wr = 0;			// Next buffer to write.  aud_rd==aud_wr means empty buffer list

#endif

//
//	WAV/OGG/MP3 input data buffering
//

 inbuf = NULL;			// Buffer for input data (as 20-bit samples)
 ib_len = 0;			// Length of input buffer (in ints)
 ib_rd = 0;			// Read-offset in inbuf
 ib_wr = 0;			// Write-offset in inbuf
 ib_eof = 0;			// End of file flag
 ib_isrun = 0;			// flag to exit the mix thread
 ib_cycle = 100;		// Time in ms for a complete loop through the buffer
 ib_read = NULL;		// Routine to refill buffer
 ib_stream_type = IBST_RAW;	// type if mix input
#ifdef WIN_MISC
 ib_hev = NULL;			// control event
 ib_het = NULL;			// read thread handle
#endif
}

/* The END...
*/

