/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is SBaGen Time computation routines

#include "sbagenr.h"

#ifdef UNIX_TIME
/* Precalculate a reference timestamp to accelerate calcNow().  This
 * can be any recent time.  We recalculate it every 10 minutes.  The
 * only reason for doing this is to cope with clocks going forwards or
 * backwards when entering or leaving summer time so that people wake
 * up on time on these two dates; an hour of the sequence will be
 * repeated or skipped.  The 'time_ref*' variables will be initialised
 * on the first call to calcNow().
 */
static int time_ref_epoch = 0;	// Reference time compared to UNIX epoch
static int time_ref_ms = 0;	// Reference time in sbagen 24-hour milliseconds

static void setupRefTime(void)
{
 struct tm *tt;
 time_t tim = time(0);

 tt = localtime(&tim);
 time_ref_epoch = tim;
 time_ref_ms = 1000 * tt -> tm_sec +
	60000 * tt -> tm_min +
	3600000 * tt -> tm_hour;
}

/* calculate current time
*/
int calcNow(void)
{
 struct timeval tv;

 if(0 != gettimeofday(&tv, 0))
  scr_error("Can't get current time");
 if(tv.tv_sec - time_ref_epoch > 600)
  setupRefTime();
 return (time_ref_ms + (tv.tv_sec - time_ref_epoch) * 1000 +
	tv.tv_usec / 1000) % H24;
}
#endif

#ifdef WIN_TIME
/* calculate current time
*/
int calcNow(void)
{
 SYSTEMTIME st;

 GetLocalTime(&st);
 return st.wMilliseconds + 1000 * st.wSecond +
	60000 * st.wMinute + 3600000 * st.wHour;
}
#endif

#if DEBUG_CHK_UTIME

/* synch. debug helper clock routine
*/
int userTime(void)
{
 struct tms buf;

 times(&buf);
 return buf.tms_utime;
}

#else

/* Dummy to avoid complaints on MSVC
*/
int userTime(void)
{
 return 0;
}

#endif

/* The END...
*/

