/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is WAW/RAW handling

#include "sbagenr.h"


/* If this is a WAV file we've been given, skip forward to the
 * 'data' section.  Don't bother checking any of the 'fmt '
 * stuff.  If they didn't give us a valid 16-bit stereo file at
 * the right rate, then tough!
 */
void find_wav_data_start(FILE *in)
{
 unsigned char buf[16];

 if(1 != fread(buf, 12, 1, in))
  goto bad;
 if(0 != memcmp(buf, "RIFF", 4))
  goto bad;
 if(0 != memcmp(buf+8, "WAVE", 4))
  goto bad;

 for(;;)
 {
  int len;

  if(1 != fread(buf, 8, 1, in))
   goto bad;
  if(0 == memcmp(buf, "data", 4))
  {
   if(-1 == (raw_init_pos = ftell(in)))
    goto bad;
   wav_data_len = buf[4] + (buf[5] << 8) + (buf[6] << 16) + (buf[7] << 24);
   return;		// We're in the right place!
  }
  len = buf[4] + (buf[5] << 8) + (buf[6] << 16) + (buf[7] << 24);
  if(len & 1)
   len++;
  if(out_rate_def && 0 == memcmp(buf, "fmt ", 4))
  {
   // Grab the sample rate to use as the default if available
   if(1 != fread(buf, 8, 1, in))
    goto bad;
   len -= 8;
   out_rate = buf[4] + (buf[5] << 8) + (buf[6] << 16) + (buf[7] << 24);
   out_rate_def = 0;
  }
  if(0 != fseek(in, len, SEEK_CUR))
   goto bad;
 }

bad:
 warn("WARNING: Not a valid WAV file, treating as RAW");
 rewind(in);
 raw_init_pos = 0;
 wav_data_len = 0;
}

/* Input raw audio data from the 'mix_in' stream, and convert to
 * 32-bit values (max 'dlen')
 */
int raw_mix_in(int *dst, int dlen)
{
 short *tmp = (void *)(dst + dlen / 2);
 int a, rv, rlen;
   
 rlen = (wav_data_len && wav_data_cnt + dlen * 2 > wav_data_len)?
  (wav_data_len - wav_data_cnt) / 2 : dlen;
 if(rlen == 0)
 {
  if(!opt_mrep)
   return 0;
  else
  {
   fseek(mix_in, raw_init_pos, SEEK_SET);
   wav_data_cnt = 0;
   rlen = dlen;
  }
 }

 for(rv = 0, a = 2; a; --a)	// 2-nd attempt -- for rewind
 {
  rv += fread(tmp + rv, 2, rlen, mix_in);
  if(rv < dlen)
  {
   if(feof(mix_in) || rlen < dlen)
   {
    if(!opt_mrep)
     break;
    else
    {
     fseek(mix_in, raw_init_pos, SEEK_SET);
     rlen = dlen - rv;
     wav_data_cnt = 0;
     continue;
    }
   }
  // we can't abort proc here, because it's call from working thread...
#ifdef UNDER_CE
   scr_fprintf(stderr, "Read error on mix input:\nCode: %08Xh\n", errno);
#else
   scr_fprintf(stderr, "Read error on mix input:\n  %s\n", strerror(errno));
#endif
   return 0;
  }
  else
   break;
 }

 if(!rv)	// EOF twice???
  return 0;

 wav_data_cnt += rlen * 2;

 // Now convert 16-bit little-endian input data into 20-bit native
 // int values
 if(bigendian)
 {
  char *rd = (void*)tmp;
  for(a = 0; a < rv; a++)
  {
   *dst++ = ((rd[0]&255) + (rd[1] << 8)) << 4;
   rd += 2;
  }
 }
 else
 {
  for(a = 0; a < rv; a++)
   *dst++ = *tmp++ << 4;
 }

 return rv;
}

/* Write a WAV header, and setup out_mode if byte-swapping is
 * required.  'byte_count' should have been set up by this point.
 */
void writeWAV(void)
{
#define addU4(xx) { int a= xx; *p++= a; *p++= (a >>= 8); *p++= (a >>= 8); *p++= (a >>= 8); }
#define addStr(xx) { char *q= xx; *p++= *q++; *p++= *q++; *p++= *q++; *p++= *q++; }

 char buf[44], *p= buf;

 if(byte_count + 36 != (int)(byte_count + 36))
 {
  int tmp;

  byte_count = 0xFFFFFFF8 - 36;
  tmp = (int)(byte_count / out_bps / out_rate);
  warn("WARNING: Selected length is too long for the WAV format; truncating to %dh%02dm%02ds",
	tmp / 3600, tmp / 60 % 60, tmp % 60);
 }

 addStr("RIFF");
 addU4((int)byte_count + 36);
 addStr("WAVE");
 addStr("fmt ");
 addU4(16);
 addU4(0x00020001);
 addU4(out_rate);
 addU4(out_rate * out_bps);
 addU4(0x0004 + 0x10000 * (out_bps * 4));	// 2,4 -> 8,16 - always assume stereo
 addStr("data");
 addU4((int)byte_count);
 writeOut(buf, 44);

 if(!opt_Q)
  warn("Outputting %d-bit WAV data at %d Hz, file size %d bytes",
	out_mode? 16 : 8, out_rate, byte_count + 44);
#undef addStr
#undef addU4
}


/* The END...
*/

